from typing import Any
from typing import Callable
from typing import Literal
from typing import Optional
from typing import Sequence
from typing import Tuple
from typing import Type
from typing import TypeVar

import cairo
from gi.repository import Atk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
from gi.repository import Gio
from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Pango

_SomeSurface = TypeVar("_SomeSurface", bound=cairo.Surface)

_namespace: str = "GtkSource"
_version: str = "4"

def completion_error_quark() -> int: ...
def encoding_get_all() -> list[Encoding]: ...
def encoding_get_current() -> Encoding: ...
def encoding_get_default_candidates() -> list[Encoding]: ...
def encoding_get_from_charset(charset: str) -> Optional[Encoding]: ...
def encoding_get_utf8() -> Encoding: ...
def file_loader_error_quark() -> int: ...
def file_saver_error_quark() -> int: ...
def finalize() -> None: ...
def init() -> None: ...
def utils_escape_search_text(text: str) -> str: ...
def utils_unescape_search_text(text: str) -> str: ...

class Buffer(Gtk.TextBuffer):
    class Props:
        can_redo: bool
        can_undo: bool
        highlight_matching_brackets: bool
        highlight_syntax: bool
        implicit_trailing_newline: bool
        language: Language
        max_undo_levels: int
        style_scheme: StyleScheme
        undo_manager: UndoManager
        copy_target_list: Gtk.TargetList
        cursor_position: int
        has_selection: bool
        paste_target_list: Gtk.TargetList
        tag_table: Gtk.TextTagTable
        text: str

    props: Props = ...
    parent_instance: Gtk.TextBuffer = ...
    priv: BufferPrivate = ...
    def __init__(
        self,
        highlight_matching_brackets: bool = ...,
        highlight_syntax: bool = ...,
        implicit_trailing_newline: bool = ...,
        language: Language = ...,
        max_undo_levels: int = ...,
        style_scheme: StyleScheme = ...,
        undo_manager: UndoManager = ...,
        tag_table: Gtk.TextTagTable = ...,
        text: str = ...,
    ): ...
    def backward_iter_to_source_mark(
        self, category: Optional[str] = None
    ) -> Tuple[bool, Gtk.TextIter]: ...
    def begin_not_undoable_action(self) -> None: ...
    def can_redo(self) -> bool: ...
    def can_undo(self) -> bool: ...
    def change_case(
        self, case_type: ChangeCaseType, start: Gtk.TextIter, end: Gtk.TextIter
    ) -> None: ...
    def create_source_mark(
        self, name: Optional[str], category: str, where: Gtk.TextIter
    ) -> Mark: ...
    def do_bracket_matched(
        self, iter: Gtk.TextIter, state: BracketMatchType
    ) -> None: ...
    def do_redo(self) -> None: ...
    def do_undo(self) -> None: ...
    def end_not_undoable_action(self) -> None: ...
    def ensure_highlight(self, start: Gtk.TextIter, end: Gtk.TextIter) -> None: ...
    def forward_iter_to_source_mark(
        self, category: Optional[str] = None
    ) -> Tuple[bool, Gtk.TextIter]: ...
    def get_context_classes_at_iter(self, iter: Gtk.TextIter) -> list[str]: ...
    def get_highlight_matching_brackets(self) -> bool: ...
    def get_highlight_syntax(self) -> bool: ...
    def get_implicit_trailing_newline(self) -> bool: ...
    def get_language(self) -> Optional[Language]: ...
    def get_max_undo_levels(self) -> int: ...
    def get_source_marks_at_iter(
        self, iter: Gtk.TextIter, category: Optional[str] = None
    ) -> list[Mark]: ...
    def get_source_marks_at_line(
        self, line: int, category: Optional[str] = None
    ) -> list[Mark]: ...
    def get_style_scheme(self) -> Optional[StyleScheme]: ...
    def get_undo_manager(self) -> Optional[UndoManager]: ...
    def iter_backward_to_context_class_toggle(
        self, context_class: str
    ) -> Tuple[bool, Gtk.TextIter]: ...
    def iter_forward_to_context_class_toggle(
        self, context_class: str
    ) -> Tuple[bool, Gtk.TextIter]: ...
    def iter_has_context_class(
        self, iter: Gtk.TextIter, context_class: str
    ) -> bool: ...
    def join_lines(self, start: Gtk.TextIter, end: Gtk.TextIter) -> None: ...
    @classmethod
    def new(cls, table: Optional[Gtk.TextTagTable] = None) -> Buffer: ...
    @classmethod
    def new_with_language(cls, language: Language) -> Buffer: ...
    def redo(self) -> None: ...
    def remove_source_marks(
        self, start: Gtk.TextIter, end: Gtk.TextIter, category: Optional[str] = None
    ) -> None: ...
    def set_highlight_matching_brackets(self, highlight: bool) -> None: ...
    def set_highlight_syntax(self, highlight: bool) -> None: ...
    def set_implicit_trailing_newline(
        self, implicit_trailing_newline: bool
    ) -> None: ...
    def set_language(self, language: Optional[Language] = None) -> None: ...
    def set_max_undo_levels(self, max_undo_levels: int) -> None: ...
    def set_style_scheme(self, scheme: Optional[StyleScheme] = None) -> None: ...
    def set_undo_manager(self, manager: Optional[UndoManager] = None) -> None: ...
    def sort_lines(
        self, start: Gtk.TextIter, end: Gtk.TextIter, flags: SortFlags, column: int
    ) -> None: ...
    def undo(self) -> None: ...

class BufferClass(GObject.GPointer):
    parent_class: Gtk.TextBufferClass = ...
    undo: Callable[[Buffer], None] = ...
    redo: Callable[[Buffer], None] = ...
    bracket_matched: Callable[[Buffer, Gtk.TextIter, BracketMatchType], None] = ...
    padding: list[None] = ...

class BufferPrivate(GObject.GPointer): ...

class Completion(GObject.Object, Gtk.Buildable):
    class Props:
        accelerators: int
        auto_complete_delay: int
        proposal_page_size: int
        provider_page_size: int
        remember_info_visibility: bool
        select_on_show: bool
        show_headers: bool
        show_icons: bool
        view: View

    props: Props = ...
    parent_instance: GObject.Object = ...
    priv: CompletionPrivate = ...
    def __init__(
        self,
        accelerators: int = ...,
        auto_complete_delay: int = ...,
        proposal_page_size: int = ...,
        provider_page_size: int = ...,
        remember_info_visibility: bool = ...,
        select_on_show: bool = ...,
        show_headers: bool = ...,
        show_icons: bool = ...,
        view: View = ...,
    ): ...
    def add_provider(self, provider: CompletionProvider) -> bool: ...
    def block_interactive(self) -> None: ...
    def create_context(
        self, position: Optional[Gtk.TextIter] = None
    ) -> CompletionContext: ...
    def do_activate_proposal(self) -> None: ...
    def do_hide(self) -> None: ...
    def do_move_cursor(self, step: Gtk.ScrollStep, num: int) -> None: ...
    def do_move_page(self, step: Gtk.ScrollStep, num: int) -> None: ...
    def do_populate_context(self, context: CompletionContext) -> None: ...
    def do_proposal_activated(
        self, provider: CompletionProvider, proposal: CompletionProposal
    ) -> bool: ...
    def do_show(self) -> None: ...
    def get_info_window(self) -> CompletionInfo: ...
    def get_providers(self) -> list[CompletionProvider]: ...
    def get_view(self) -> Optional[View]: ...
    def hide(self) -> None: ...
    def remove_provider(self, provider: CompletionProvider) -> bool: ...
    def start(
        self, providers: Optional[list[CompletionProvider]], context: CompletionContext
    ) -> bool: ...
    def unblock_interactive(self) -> None: ...

class CompletionClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    proposal_activated: Callable[
        [Completion, CompletionProvider, CompletionProposal], bool
    ] = ...
    show: Callable[[Completion], None] = ...
    hide: Callable[[Completion], None] = ...
    populate_context: Callable[[Completion, CompletionContext], None] = ...
    move_cursor: Callable[[Completion, Gtk.ScrollStep, int], None] = ...
    move_page: Callable[[Completion, Gtk.ScrollStep, int], None] = ...
    activate_proposal: Callable[[Completion], None] = ...
    padding: list[None] = ...

class CompletionContext(GObject.InitiallyUnowned):
    class Props:
        activation: CompletionActivation
        completion: Completion
        iter: Gtk.TextIter

    props: Props = ...
    parent: GObject.InitiallyUnowned = ...
    priv: CompletionContextPrivate = ...
    def __init__(
        self,
        activation: CompletionActivation = ...,
        completion: Completion = ...,
        iter: Gtk.TextIter = ...,
    ): ...
    def add_proposals(
        self,
        provider: CompletionProvider,
        proposals: Optional[list[CompletionProposal]],
        finished: bool,
    ) -> None: ...
    def do_cancelled(self) -> None: ...
    def get_activation(self) -> CompletionActivation: ...
    def get_iter(self) -> Tuple[bool, Gtk.TextIter]: ...

class CompletionContextClass(GObject.GPointer):
    parent_class: GObject.InitiallyUnownedClass = ...
    cancelled: Callable[[CompletionContext], None] = ...
    padding: list[None] = ...

class CompletionContextPrivate(GObject.GPointer): ...

class CompletionInfo(Gtk.Window, Atk.ImplementorIface, Gtk.Buildable):
    class Props:
        accept_focus: bool
        application: Gtk.Application
        attached_to: Gtk.Widget
        decorated: bool
        default_height: int
        default_width: int
        deletable: bool
        destroy_with_parent: bool
        focus_on_map: bool
        focus_visible: bool
        gravity: Gdk.Gravity
        has_resize_grip: bool
        has_toplevel_focus: bool
        hide_titlebar_when_maximized: bool
        icon: GdkPixbuf.Pixbuf
        icon_name: str
        is_active: bool
        is_maximized: bool
        mnemonics_visible: bool
        modal: bool
        resizable: bool
        resize_grip_visible: bool
        role: str
        screen: Gdk.Screen
        skip_pager_hint: bool
        skip_taskbar_hint: bool
        startup_id: str
        title: str
        transient_for: Gtk.Window
        type: Gtk.WindowType
        type_hint: Gdk.WindowTypeHint
        urgency_hint: bool
        window_position: Gtk.WindowPosition
        border_width: int
        child: Gtk.Widget
        resize_mode: Gtk.ResizeMode
        app_paintable: bool
        can_default: bool
        can_focus: bool
        composite_child: bool
        double_buffered: bool
        events: Gdk.EventMask
        expand: bool
        focus_on_click: bool
        halign: Gtk.Align
        has_default: bool
        has_focus: bool
        has_tooltip: bool
        height_request: int
        hexpand: bool
        hexpand_set: bool
        is_focus: bool
        margin: int
        margin_bottom: int
        margin_end: int
        margin_left: int
        margin_right: int
        margin_start: int
        margin_top: int
        name: str
        no_show_all: bool
        opacity: float
        parent: Gtk.Container
        receives_default: bool
        scale_factor: int
        sensitive: bool
        style: Gtk.Style
        tooltip_markup: str
        tooltip_text: str
        valign: Gtk.Align
        vexpand: bool
        vexpand_set: bool
        visible: bool
        width_request: int
        window: Gdk.Window

    props: Props = ...
    parent: Gtk.Window = ...
    priv: CompletionInfoPrivate = ...
    def __init__(
        self,
        accept_focus: bool = ...,
        application: Gtk.Application = ...,
        attached_to: Gtk.Widget = ...,
        decorated: bool = ...,
        default_height: int = ...,
        default_width: int = ...,
        deletable: bool = ...,
        destroy_with_parent: bool = ...,
        focus_on_map: bool = ...,
        focus_visible: bool = ...,
        gravity: Gdk.Gravity = ...,
        has_resize_grip: bool = ...,
        hide_titlebar_when_maximized: bool = ...,
        icon: GdkPixbuf.Pixbuf = ...,
        icon_name: str = ...,
        mnemonics_visible: bool = ...,
        modal: bool = ...,
        resizable: bool = ...,
        role: str = ...,
        screen: Gdk.Screen = ...,
        skip_pager_hint: bool = ...,
        skip_taskbar_hint: bool = ...,
        startup_id: str = ...,
        title: str = ...,
        transient_for: Gtk.Window = ...,
        type: Gtk.WindowType = ...,
        type_hint: Gdk.WindowTypeHint = ...,
        urgency_hint: bool = ...,
        window_position: Gtk.WindowPosition = ...,
        border_width: int = ...,
        child: Gtk.Widget = ...,
        resize_mode: Gtk.ResizeMode = ...,
        app_paintable: bool = ...,
        can_default: bool = ...,
        can_focus: bool = ...,
        double_buffered: bool = ...,
        events: Gdk.EventMask = ...,
        expand: bool = ...,
        focus_on_click: bool = ...,
        halign: Gtk.Align = ...,
        has_default: bool = ...,
        has_focus: bool = ...,
        has_tooltip: bool = ...,
        height_request: int = ...,
        hexpand: bool = ...,
        hexpand_set: bool = ...,
        is_focus: bool = ...,
        margin: int = ...,
        margin_bottom: int = ...,
        margin_end: int = ...,
        margin_left: int = ...,
        margin_right: int = ...,
        margin_start: int = ...,
        margin_top: int = ...,
        name: str = ...,
        no_show_all: bool = ...,
        opacity: float = ...,
        parent: Gtk.Container = ...,
        receives_default: bool = ...,
        sensitive: bool = ...,
        style: Gtk.Style = ...,
        tooltip_markup: str = ...,
        tooltip_text: str = ...,
        valign: Gtk.Align = ...,
        vexpand: bool = ...,
        vexpand_set: bool = ...,
        visible: bool = ...,
        width_request: int = ...,
    ): ...
    def move_to_iter(
        self, view: Gtk.TextView, iter: Optional[Gtk.TextIter] = None
    ) -> None: ...
    @classmethod
    def new(cls) -> CompletionInfo: ...

class CompletionInfoClass(GObject.GPointer):
    parent_class: Gtk.WindowClass = ...
    padding: list[None] = ...

class CompletionInfoPrivate(GObject.GPointer): ...

class CompletionItem(GObject.Object, CompletionProposal):
    class Props:
        gicon: Gio.Icon
        icon: GdkPixbuf.Pixbuf
        icon_name: str
        info: str
        label: str
        markup: str
        text: str

    props: Props = ...
    parent: GObject.Object = ...
    priv: CompletionItemPrivate = ...
    def __init__(
        self,
        gicon: Gio.Icon = ...,
        icon: GdkPixbuf.Pixbuf = ...,
        icon_name: str = ...,
        info: str = ...,
        label: str = ...,
        markup: str = ...,
        text: str = ...,
    ): ...
    @classmethod
    def new(cls) -> CompletionItem: ...
    def set_gicon(self, gicon: Optional[Gio.Icon] = None) -> None: ...
    def set_icon(self, icon: Optional[GdkPixbuf.Pixbuf] = None) -> None: ...
    def set_icon_name(self, icon_name: Optional[str] = None) -> None: ...
    def set_info(self, info: Optional[str] = None) -> None: ...
    def set_label(self, label: Optional[str] = None) -> None: ...
    def set_markup(self, markup: Optional[str] = None) -> None: ...
    def set_text(self, text: Optional[str] = None) -> None: ...

class CompletionItemClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class CompletionItemPrivate(GObject.GPointer): ...
class CompletionPrivate(GObject.GPointer): ...

class CompletionProposal(GObject.GInterface):
    def changed(self) -> None: ...
    def equal(self, other: CompletionProposal) -> bool: ...
    def get_gicon(self) -> Optional[Gio.Icon]: ...
    def get_icon(self) -> Optional[GdkPixbuf.Pixbuf]: ...
    def get_icon_name(self) -> Optional[str]: ...
    def get_info(self) -> Optional[str]: ...
    def get_label(self) -> str: ...
    def get_markup(self) -> str: ...
    def get_text(self) -> str: ...
    def hash(self) -> int: ...

class CompletionProposalIface(GObject.GPointer):
    parent: GObject.TypeInterface = ...
    get_label: Callable[[CompletionProposal], str] = ...
    get_markup: Callable[[CompletionProposal], str] = ...
    get_text: Callable[[CompletionProposal], str] = ...
    get_icon: Callable[[CompletionProposal], Optional[GdkPixbuf.Pixbuf]] = ...
    get_icon_name: Callable[[CompletionProposal], Optional[str]] = ...
    get_gicon: Callable[[CompletionProposal], Optional[Gio.Icon]] = ...
    get_info: Callable[[CompletionProposal], Optional[str]] = ...
    hash: Callable[[CompletionProposal], int] = ...
    equal: Callable[[CompletionProposal, CompletionProposal], bool] = ...
    changed: Callable[[CompletionProposal], None] = ...

class CompletionProvider(GObject.GInterface):
    def activate_proposal(
        self, proposal: CompletionProposal, iter: Gtk.TextIter
    ) -> bool: ...
    def get_activation(self) -> CompletionActivation: ...
    def get_gicon(self) -> Optional[Gio.Icon]: ...
    def get_icon(self) -> Optional[GdkPixbuf.Pixbuf]: ...
    def get_icon_name(self) -> Optional[str]: ...
    def get_info_widget(self, proposal: CompletionProposal) -> Optional[Gtk.Widget]: ...
    def get_interactive_delay(self) -> int: ...
    def get_name(self) -> str: ...
    def get_priority(self) -> int: ...
    def get_start_iter(
        self, context: CompletionContext, proposal: CompletionProposal
    ) -> Tuple[bool, Gtk.TextIter]: ...
    def match(self, context: CompletionContext) -> bool: ...
    def populate(self, context: CompletionContext) -> None: ...
    def update_info(
        self, proposal: CompletionProposal, info: CompletionInfo
    ) -> None: ...

class CompletionProviderIface(GObject.GPointer):
    g_iface: GObject.TypeInterface = ...
    get_name: Callable[[CompletionProvider], str] = ...
    get_icon: Callable[[CompletionProvider], Optional[GdkPixbuf.Pixbuf]] = ...
    get_icon_name: Callable[[CompletionProvider], Optional[str]] = ...
    get_gicon: Callable[[CompletionProvider], Optional[Gio.Icon]] = ...
    populate: Callable[[CompletionProvider, CompletionContext], None] = ...
    match: Callable[[CompletionProvider, CompletionContext], bool] = ...
    get_activation: Callable[[CompletionProvider], CompletionActivation] = ...
    get_info_widget: Callable[
        [CompletionProvider, CompletionProposal], Optional[Gtk.Widget]
    ] = ...
    update_info: Callable[
        [CompletionProvider, CompletionProposal, CompletionInfo], None
    ] = ...
    get_start_iter: Callable[
        [CompletionProvider, CompletionContext, CompletionProposal],
        Tuple[bool, Gtk.TextIter],
    ] = ...
    activate_proposal: Callable[
        [CompletionProvider, CompletionProposal, Gtk.TextIter], bool
    ] = ...
    get_interactive_delay: Callable[[CompletionProvider], int] = ...
    get_priority: Callable[[CompletionProvider], int] = ...

class CompletionWords(GObject.Object, CompletionProvider):
    class Props:
        activation: CompletionActivation
        icon: GdkPixbuf.Pixbuf
        interactive_delay: int
        minimum_word_size: int
        name: str
        priority: int
        proposals_batch_size: int
        scan_batch_size: int

    props: Props = ...
    parent: GObject.Object = ...
    priv: CompletionWordsPrivate = ...
    def __init__(
        self,
        activation: CompletionActivation = ...,
        icon: GdkPixbuf.Pixbuf = ...,
        interactive_delay: int = ...,
        minimum_word_size: int = ...,
        name: str = ...,
        priority: int = ...,
        proposals_batch_size: int = ...,
        scan_batch_size: int = ...,
    ): ...
    @classmethod
    def new(
        cls, name: Optional[str] = None, icon: Optional[GdkPixbuf.Pixbuf] = None
    ) -> CompletionWords: ...
    def register(self, buffer: Gtk.TextBuffer) -> None: ...
    def unregister(self, buffer: Gtk.TextBuffer) -> None: ...

class CompletionWordsClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...

class CompletionWordsPrivate(GObject.GPointer): ...

class Encoding(GObject.GBoxed):
    def copy(self) -> Encoding: ...
    def free(self) -> None: ...
    @staticmethod
    def get_all() -> list[Encoding]: ...
    def get_charset(self) -> str: ...
    @staticmethod
    def get_current() -> Encoding: ...
    @staticmethod
    def get_default_candidates() -> list[Encoding]: ...
    @staticmethod
    def get_from_charset(charset: str) -> Optional[Encoding]: ...
    def get_name(self) -> str: ...
    @staticmethod
    def get_utf8() -> Encoding: ...
    def to_string(self) -> str: ...

class File(GObject.Object):
    class Props:
        compression_type: CompressionType
        encoding: Encoding
        location: Gio.File
        newline_type: NewlineType
        read_only: bool

    props: Props = ...
    parent: GObject.Object = ...
    priv: FilePrivate = ...
    def __init__(self, location: Gio.File = ...): ...
    def check_file_on_disk(self) -> None: ...
    def get_compression_type(self) -> CompressionType: ...
    def get_encoding(self) -> Encoding: ...
    def get_location(self) -> Gio.File: ...
    def get_newline_type(self) -> NewlineType: ...
    def is_deleted(self) -> bool: ...
    def is_externally_modified(self) -> bool: ...
    def is_local(self) -> bool: ...
    def is_readonly(self) -> bool: ...
    @classmethod
    def new(cls) -> File: ...
    def set_location(self, location: Optional[Gio.File] = None) -> None: ...

class FileClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class FileLoader(GObject.Object):
    class Props:
        buffer: Buffer
        file: File
        input_stream: Gio.InputStream
        location: Gio.File

    props: Props = ...
    parent: GObject.Object = ...
    priv: FileLoaderPrivate = ...
    def __init__(
        self,
        buffer: Buffer = ...,
        file: File = ...,
        input_stream: Gio.InputStream = ...,
        location: Gio.File = ...,
    ): ...
    def get_buffer(self) -> Buffer: ...
    def get_compression_type(self) -> CompressionType: ...
    def get_encoding(self) -> Encoding: ...
    def get_file(self) -> File: ...
    def get_input_stream(self) -> Optional[Gio.InputStream]: ...
    def get_location(self) -> Optional[Gio.File]: ...
    def get_newline_type(self) -> NewlineType: ...
    def load_async(
        self,
        io_priority: int,
        cancellable: Optional[Gio.Cancellable] = None,
        progress_callback: Optional[Callable[..., None]] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def load_finish(self, result: Gio.AsyncResult) -> bool: ...
    @classmethod
    def new(cls, buffer: Buffer, file: File) -> FileLoader: ...
    @classmethod
    def new_from_stream(
        cls, buffer: Buffer, file: File, stream: Gio.InputStream
    ) -> FileLoader: ...
    def set_candidate_encodings(self, candidate_encodings: list[Encoding]) -> None: ...

class FileLoaderClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class FileLoaderPrivate(GObject.GPointer): ...
class FilePrivate(GObject.GPointer): ...

class FileSaver(GObject.Object):
    class Props:
        buffer: Buffer
        compression_type: CompressionType
        encoding: Encoding
        file: File
        flags: FileSaverFlags
        location: Gio.File
        newline_type: NewlineType

    props: Props = ...
    object: GObject.Object = ...
    priv: FileSaverPrivate = ...
    def __init__(
        self,
        buffer: Buffer = ...,
        compression_type: CompressionType = ...,
        encoding: Encoding = ...,
        file: File = ...,
        flags: FileSaverFlags = ...,
        location: Gio.File = ...,
        newline_type: NewlineType = ...,
    ): ...
    def get_buffer(self) -> Buffer: ...
    def get_compression_type(self) -> CompressionType: ...
    def get_encoding(self) -> Encoding: ...
    def get_file(self) -> File: ...
    def get_flags(self) -> FileSaverFlags: ...
    def get_location(self) -> Gio.File: ...
    def get_newline_type(self) -> NewlineType: ...
    @classmethod
    def new(cls, buffer: Buffer, file: File) -> FileSaver: ...
    @classmethod
    def new_with_target(
        cls, buffer: Buffer, file: File, target_location: Gio.File
    ) -> FileSaver: ...
    def save_async(
        self,
        io_priority: int,
        cancellable: Optional[Gio.Cancellable] = None,
        progress_callback: Optional[Callable[..., None]] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def save_finish(self, result: Gio.AsyncResult) -> bool: ...
    def set_compression_type(self, compression_type: CompressionType) -> None: ...
    def set_encoding(self, encoding: Optional[Encoding] = None) -> None: ...
    def set_flags(self, flags: FileSaverFlags) -> None: ...
    def set_newline_type(self, newline_type: NewlineType) -> None: ...

class FileSaverClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class FileSaverPrivate(GObject.GPointer): ...

class Gutter(GObject.Object):
    class Props:
        view: View
        window_type: Gtk.TextWindowType

    props: Props = ...
    parent: GObject.Object = ...
    priv: GutterPrivate = ...
    def __init__(self, view: View = ..., window_type: Gtk.TextWindowType = ...): ...
    def get_renderer_at_pos(self, x: int, y: int) -> Optional[GutterRenderer]: ...
    def get_view(self) -> View: ...
    def get_window_type(self) -> Gtk.TextWindowType: ...
    def insert(self, renderer: GutterRenderer, position: int) -> bool: ...
    def queue_draw(self) -> None: ...
    def remove(self, renderer: GutterRenderer) -> None: ...
    def reorder(self, renderer: GutterRenderer, position: int) -> None: ...

class GutterClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class GutterPrivate(GObject.GPointer): ...

class GutterRenderer(GObject.InitiallyUnowned):
    class Props:
        alignment_mode: GutterRendererAlignmentMode
        background_rgba: Gdk.RGBA
        background_set: bool
        size: int
        view: Gtk.TextView
        visible: bool
        window_type: Gtk.TextWindowType
        xalign: float
        xpad: int
        yalign: float
        ypad: int

    props: Props = ...
    parent: GObject.InitiallyUnowned = ...
    priv: GutterRendererPrivate = ...
    def __init__(
        self,
        alignment_mode: GutterRendererAlignmentMode = ...,
        background_rgba: Gdk.RGBA = ...,
        background_set: bool = ...,
        size: int = ...,
        visible: bool = ...,
        xalign: float = ...,
        xpad: int = ...,
        yalign: float = ...,
        ypad: int = ...,
    ): ...
    def activate(
        self, iter: Gtk.TextIter, area: Gdk.Rectangle, event: Gdk.Event
    ) -> None: ...
    def begin(
        self,
        cr: cairo.Context[_SomeSurface],
        background_area: Gdk.Rectangle,
        cell_area: Gdk.Rectangle,
        start: Gtk.TextIter,
        end: Gtk.TextIter,
    ) -> None: ...
    def do_activate(
        self, iter: Gtk.TextIter, area: Gdk.Rectangle, event: Gdk.Event
    ) -> None: ...
    def do_begin(
        self,
        cr: cairo.Context[_SomeSurface],
        background_area: Gdk.Rectangle,
        cell_area: Gdk.Rectangle,
        start: Gtk.TextIter,
        end: Gtk.TextIter,
    ) -> None: ...
    def do_change_buffer(self, old_buffer: Optional[Gtk.TextBuffer] = None) -> None: ...
    def do_change_view(self, old_view: Optional[Gtk.TextView] = None) -> None: ...
    def do_draw(
        self,
        cr: cairo.Context[_SomeSurface],
        background_area: Gdk.Rectangle,
        cell_area: Gdk.Rectangle,
        start: Gtk.TextIter,
        end: Gtk.TextIter,
        state: GutterRendererState,
    ) -> None: ...
    def do_end(self) -> None: ...
    def do_query_activatable(
        self, iter: Gtk.TextIter, area: Gdk.Rectangle, event: Gdk.Event
    ) -> bool: ...
    def do_query_data(
        self, start: Gtk.TextIter, end: Gtk.TextIter, state: GutterRendererState
    ) -> None: ...
    def do_query_tooltip(
        self,
        iter: Gtk.TextIter,
        area: Gdk.Rectangle,
        x: int,
        y: int,
        tooltip: Gtk.Tooltip,
    ) -> bool: ...
    def do_queue_draw(self) -> None: ...
    def draw(
        self,
        cr: cairo.Context[_SomeSurface],
        background_area: Gdk.Rectangle,
        cell_area: Gdk.Rectangle,
        start: Gtk.TextIter,
        end: Gtk.TextIter,
        state: GutterRendererState,
    ) -> None: ...
    def end(self) -> None: ...
    def get_alignment(self) -> Tuple[float, float]: ...
    def get_alignment_mode(self) -> GutterRendererAlignmentMode: ...
    def get_background(self) -> Tuple[bool, Gdk.RGBA]: ...
    def get_padding(self) -> Tuple[int, int]: ...
    def get_size(self) -> int: ...
    def get_view(self) -> Gtk.TextView: ...
    def get_visible(self) -> bool: ...
    def get_window_type(self) -> Gtk.TextWindowType: ...
    def query_activatable(
        self, iter: Gtk.TextIter, area: Gdk.Rectangle, event: Gdk.Event
    ) -> bool: ...
    def query_data(
        self, start: Gtk.TextIter, end: Gtk.TextIter, state: GutterRendererState
    ) -> None: ...
    def query_tooltip(
        self,
        iter: Gtk.TextIter,
        area: Gdk.Rectangle,
        x: int,
        y: int,
        tooltip: Gtk.Tooltip,
    ) -> bool: ...
    def queue_draw(self) -> None: ...
    def set_alignment(self, xalign: float, yalign: float) -> None: ...
    def set_alignment_mode(self, mode: GutterRendererAlignmentMode) -> None: ...
    def set_background(self, color: Optional[Gdk.RGBA] = None) -> None: ...
    def set_padding(self, xpad: int, ypad: int) -> None: ...
    def set_size(self, size: int) -> None: ...
    def set_visible(self, visible: bool) -> None: ...

class GutterRendererClass(GObject.GPointer):
    parent_class: GObject.InitiallyUnownedClass = ...
    begin: Callable[
        [
            GutterRenderer,
            cairo.Context[_SomeSurface],
            Gdk.Rectangle,
            Gdk.Rectangle,
            Gtk.TextIter,
            Gtk.TextIter,
        ],
        None,
    ] = ...
    draw: Callable[
        [
            GutterRenderer,
            cairo.Context[_SomeSurface],
            Gdk.Rectangle,
            Gdk.Rectangle,
            Gtk.TextIter,
            Gtk.TextIter,
            GutterRendererState,
        ],
        None,
    ] = ...
    end: Callable[[GutterRenderer], None] = ...
    change_view: Callable[[GutterRenderer, Optional[Gtk.TextView]], None] = ...
    change_buffer: Callable[[GutterRenderer, Optional[Gtk.TextBuffer]], None] = ...
    query_activatable: Callable[
        [GutterRenderer, Gtk.TextIter, Gdk.Rectangle, Gdk.Event], bool
    ] = ...
    activate: Callable[
        [GutterRenderer, Gtk.TextIter, Gdk.Rectangle, Gdk.Event], None
    ] = ...
    queue_draw: Callable[[GutterRenderer], None] = ...
    query_tooltip: Callable[
        [GutterRenderer, Gtk.TextIter, Gdk.Rectangle, int, int, Gtk.Tooltip], bool
    ] = ...
    query_data: Callable[
        [GutterRenderer, Gtk.TextIter, Gtk.TextIter, GutterRendererState], None
    ] = ...
    padding: list[None] = ...

class GutterRendererPixbuf(GutterRenderer):
    class Props:
        gicon: Gio.Icon
        icon_name: str
        pixbuf: GdkPixbuf.Pixbuf
        alignment_mode: GutterRendererAlignmentMode
        background_rgba: Gdk.RGBA
        background_set: bool
        size: int
        view: Gtk.TextView
        visible: bool
        window_type: Gtk.TextWindowType
        xalign: float
        xpad: int
        yalign: float
        ypad: int

    props: Props = ...
    parent: GutterRenderer = ...
    priv: GutterRendererPixbufPrivate = ...
    def __init__(
        self,
        gicon: Gio.Icon = ...,
        icon_name: str = ...,
        pixbuf: GdkPixbuf.Pixbuf = ...,
        alignment_mode: GutterRendererAlignmentMode = ...,
        background_rgba: Gdk.RGBA = ...,
        background_set: bool = ...,
        size: int = ...,
        visible: bool = ...,
        xalign: float = ...,
        xpad: int = ...,
        yalign: float = ...,
        ypad: int = ...,
    ): ...
    def get_gicon(self) -> Gio.Icon: ...
    def get_icon_name(self) -> str: ...
    def get_pixbuf(self) -> GdkPixbuf.Pixbuf: ...
    @classmethod
    def new(cls) -> GutterRendererPixbuf: ...
    def set_gicon(self, icon: Optional[Gio.Icon] = None) -> None: ...
    def set_icon_name(self, icon_name: Optional[str] = None) -> None: ...
    def set_pixbuf(self, pixbuf: Optional[GdkPixbuf.Pixbuf] = None) -> None: ...

class GutterRendererPixbufClass(GObject.GPointer):
    parent_class: GutterRendererClass = ...
    padding: list[None] = ...

class GutterRendererPixbufPrivate(GObject.GPointer): ...
class GutterRendererPrivate(GObject.GPointer): ...

class GutterRendererText(GutterRenderer):
    class Props:
        markup: str
        text: str
        alignment_mode: GutterRendererAlignmentMode
        background_rgba: Gdk.RGBA
        background_set: bool
        size: int
        view: Gtk.TextView
        visible: bool
        window_type: Gtk.TextWindowType
        xalign: float
        xpad: int
        yalign: float
        ypad: int

    props: Props = ...
    parent: GutterRenderer = ...
    priv: GutterRendererTextPrivate = ...
    def __init__(
        self,
        markup: str = ...,
        text: str = ...,
        alignment_mode: GutterRendererAlignmentMode = ...,
        background_rgba: Gdk.RGBA = ...,
        background_set: bool = ...,
        size: int = ...,
        visible: bool = ...,
        xalign: float = ...,
        xpad: int = ...,
        yalign: float = ...,
        ypad: int = ...,
    ): ...
    def measure(self, text: str) -> Tuple[int, int]: ...
    def measure_markup(self, markup: str) -> Tuple[int, int]: ...
    @classmethod
    def new(cls) -> GutterRendererText: ...
    def set_markup(self, markup: str, length: int) -> None: ...
    def set_text(self, text: str, length: int) -> None: ...

class GutterRendererTextClass(GObject.GPointer):
    parent_class: GutterRendererClass = ...
    padding: list[None] = ...

class GutterRendererTextPrivate(GObject.GPointer): ...

class Language(GObject.Object):
    class Props:
        hidden: bool
        id: str
        name: str
        section: str

    props: Props = ...
    parent_instance: GObject.Object = ...
    priv: LanguagePrivate = ...
    def get_globs(self) -> Optional[list[str]]: ...
    def get_hidden(self) -> bool: ...
    def get_id(self) -> str: ...
    def get_metadata(self, name: str) -> Optional[str]: ...
    def get_mime_types(self) -> Optional[list[str]]: ...
    def get_name(self) -> str: ...
    def get_section(self) -> str: ...
    def get_style_fallback(self, style_id: str) -> Optional[str]: ...
    def get_style_ids(self) -> Optional[list[str]]: ...
    def get_style_name(self, style_id: str) -> Optional[str]: ...

class LanguageClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class LanguageManager(GObject.Object):
    class Props:
        language_ids: list[str]
        search_path: list[str]

    props: Props = ...
    parent_instance: GObject.Object = ...
    priv: LanguageManagerPrivate = ...
    def __init__(self, search_path: Sequence[str] = ...): ...
    @staticmethod
    def get_default() -> LanguageManager: ...
    def get_language(self, id: str) -> Optional[Language]: ...
    def get_language_ids(self) -> Optional[list[str]]: ...
    def get_search_path(self) -> list[str]: ...
    def guess_language(
        self, filename: Optional[str] = None, content_type: Optional[str] = None
    ) -> Optional[Language]: ...
    @classmethod
    def new(cls) -> LanguageManager: ...
    def set_search_path(self, dirs: Optional[Sequence[str]] = None) -> None: ...

class LanguageManagerClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class LanguageManagerPrivate(GObject.GPointer): ...
class LanguagePrivate(GObject.GPointer): ...

class Map(View, Atk.ImplementorIface, Gtk.Buildable, Gtk.Scrollable):
    class Props:
        font_desc: Pango.FontDescription
        view: View
        auto_indent: bool
        background_pattern: BackgroundPatternType
        completion: Completion
        highlight_current_line: bool
        indent_on_tab: bool
        indent_width: int
        insert_spaces_instead_of_tabs: bool
        right_margin_position: int
        show_line_marks: bool
        show_line_numbers: bool
        show_right_margin: bool
        smart_backspace: bool
        smart_home_end: SmartHomeEndType
        space_drawer: SpaceDrawer
        tab_width: int
        accepts_tab: bool
        bottom_margin: int
        buffer: Gtk.TextBuffer
        cursor_visible: bool
        editable: bool
        im_module: str
        indent: int
        input_hints: Gtk.InputHints
        input_purpose: Gtk.InputPurpose
        justification: Gtk.Justification
        left_margin: int
        monospace: bool
        overwrite: bool
        pixels_above_lines: int
        pixels_below_lines: int
        pixels_inside_wrap: int
        populate_all: bool
        right_margin: int
        tabs: Pango.TabArray
        top_margin: int
        wrap_mode: Gtk.WrapMode
        border_width: int
        child: Gtk.Widget
        resize_mode: Gtk.ResizeMode
        app_paintable: bool
        can_default: bool
        can_focus: bool
        composite_child: bool
        double_buffered: bool
        events: Gdk.EventMask
        expand: bool
        focus_on_click: bool
        halign: Gtk.Align
        has_default: bool
        has_focus: bool
        has_tooltip: bool
        height_request: int
        hexpand: bool
        hexpand_set: bool
        is_focus: bool
        margin: int
        margin_bottom: int
        margin_end: int
        margin_left: int
        margin_right: int
        margin_start: int
        margin_top: int
        name: str
        no_show_all: bool
        opacity: float
        parent: Gtk.Container
        receives_default: bool
        scale_factor: int
        sensitive: bool
        style: Gtk.Style
        tooltip_markup: str
        tooltip_text: str
        valign: Gtk.Align
        vexpand: bool
        vexpand_set: bool
        visible: bool
        width_request: int
        window: Gdk.Window
        hadjustment: Gtk.Adjustment
        hscroll_policy: Gtk.ScrollablePolicy
        vadjustment: Gtk.Adjustment
        vscroll_policy: Gtk.ScrollablePolicy

    props: Props = ...
    parent_instance: View = ...
    def __init__(
        self,
        font_desc: Pango.FontDescription = ...,
        view: View = ...,
        auto_indent: bool = ...,
        background_pattern: BackgroundPatternType = ...,
        highlight_current_line: bool = ...,
        indent_on_tab: bool = ...,
        indent_width: int = ...,
        insert_spaces_instead_of_tabs: bool = ...,
        right_margin_position: int = ...,
        show_line_marks: bool = ...,
        show_line_numbers: bool = ...,
        show_right_margin: bool = ...,
        smart_backspace: bool = ...,
        smart_home_end: SmartHomeEndType = ...,
        tab_width: int = ...,
        accepts_tab: bool = ...,
        bottom_margin: int = ...,
        buffer: Gtk.TextBuffer = ...,
        cursor_visible: bool = ...,
        editable: bool = ...,
        im_module: str = ...,
        indent: int = ...,
        input_hints: Gtk.InputHints = ...,
        input_purpose: Gtk.InputPurpose = ...,
        justification: Gtk.Justification = ...,
        left_margin: int = ...,
        monospace: bool = ...,
        overwrite: bool = ...,
        pixels_above_lines: int = ...,
        pixels_below_lines: int = ...,
        pixels_inside_wrap: int = ...,
        populate_all: bool = ...,
        right_margin: int = ...,
        tabs: Pango.TabArray = ...,
        top_margin: int = ...,
        wrap_mode: Gtk.WrapMode = ...,
        border_width: int = ...,
        child: Gtk.Widget = ...,
        resize_mode: Gtk.ResizeMode = ...,
        app_paintable: bool = ...,
        can_default: bool = ...,
        can_focus: bool = ...,
        double_buffered: bool = ...,
        events: Gdk.EventMask = ...,
        expand: bool = ...,
        focus_on_click: bool = ...,
        halign: Gtk.Align = ...,
        has_default: bool = ...,
        has_focus: bool = ...,
        has_tooltip: bool = ...,
        height_request: int = ...,
        hexpand: bool = ...,
        hexpand_set: bool = ...,
        is_focus: bool = ...,
        margin: int = ...,
        margin_bottom: int = ...,
        margin_end: int = ...,
        margin_left: int = ...,
        margin_right: int = ...,
        margin_start: int = ...,
        margin_top: int = ...,
        name: str = ...,
        no_show_all: bool = ...,
        opacity: float = ...,
        parent: Gtk.Container = ...,
        receives_default: bool = ...,
        sensitive: bool = ...,
        style: Gtk.Style = ...,
        tooltip_markup: str = ...,
        tooltip_text: str = ...,
        valign: Gtk.Align = ...,
        vexpand: bool = ...,
        vexpand_set: bool = ...,
        visible: bool = ...,
        width_request: int = ...,
        hadjustment: Gtk.Adjustment = ...,
        hscroll_policy: Gtk.ScrollablePolicy = ...,
        vadjustment: Gtk.Adjustment = ...,
        vscroll_policy: Gtk.ScrollablePolicy = ...,
    ): ...
    def get_view(self) -> Optional[View]: ...
    @classmethod
    def new(cls) -> Map: ...
    def set_view(self, view: View) -> None: ...

class MapClass(GObject.GPointer):
    parent_class: ViewClass = ...
    padding: list[None] = ...

class Mark(Gtk.TextMark):
    class Props:
        category: str
        left_gravity: bool
        name: str

    props: Props = ...
    parent_instance: Gtk.TextMark = ...
    priv: MarkPrivate = ...
    def __init__(
        self, category: str = ..., left_gravity: bool = ..., name: str = ...
    ): ...
    def get_category(self) -> str: ...
    @classmethod
    def new(cls, name: Optional[str], category: str) -> Mark: ...
    def next(self, category: Optional[str] = None) -> Optional[Mark]: ...
    def prev(self, category: str) -> Optional[Mark]: ...

class MarkAttributes(GObject.Object):
    class Props:
        background: Gdk.RGBA
        gicon: Gio.Icon
        icon_name: str
        pixbuf: GdkPixbuf.Pixbuf

    props: Props = ...
    parent: GObject.Object = ...
    priv: MarkAttributesPrivate = ...
    def __init__(
        self,
        background: Gdk.RGBA = ...,
        gicon: Gio.Icon = ...,
        icon_name: str = ...,
        pixbuf: GdkPixbuf.Pixbuf = ...,
    ): ...
    def get_background(self) -> Tuple[bool, Gdk.RGBA]: ...
    def get_gicon(self) -> Gio.Icon: ...
    def get_icon_name(self) -> str: ...
    def get_pixbuf(self) -> GdkPixbuf.Pixbuf: ...
    def get_tooltip_markup(self, mark: Mark) -> str: ...
    def get_tooltip_text(self, mark: Mark) -> str: ...
    @classmethod
    def new(cls) -> MarkAttributes: ...
    def render_icon(self, widget: Gtk.Widget, size: int) -> GdkPixbuf.Pixbuf: ...
    def set_background(self, background: Gdk.RGBA) -> None: ...
    def set_gicon(self, gicon: Gio.Icon) -> None: ...
    def set_icon_name(self, icon_name: str) -> None: ...
    def set_pixbuf(self, pixbuf: GdkPixbuf.Pixbuf) -> None: ...

class MarkAttributesClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class MarkAttributesPrivate(GObject.GPointer): ...

class MarkClass(GObject.GPointer):
    parent_class: Gtk.TextMarkClass = ...
    padding: list[None] = ...

class MarkPrivate(GObject.GPointer): ...

class PrintCompositor(GObject.Object):
    class Props:
        body_font_name: str
        buffer: Buffer
        footer_font_name: str
        header_font_name: str
        highlight_syntax: bool
        line_numbers_font_name: str
        n_pages: int
        print_footer: bool
        print_header: bool
        print_line_numbers: int
        tab_width: int
        wrap_mode: Gtk.WrapMode

    props: Props = ...
    parent_instance: GObject.Object = ...
    priv: PrintCompositorPrivate = ...
    def __init__(
        self,
        body_font_name: str = ...,
        buffer: Buffer = ...,
        footer_font_name: str = ...,
        header_font_name: str = ...,
        highlight_syntax: bool = ...,
        line_numbers_font_name: str = ...,
        print_footer: bool = ...,
        print_header: bool = ...,
        print_line_numbers: int = ...,
        tab_width: int = ...,
        wrap_mode: Gtk.WrapMode = ...,
    ): ...
    def draw_page(self, context: Gtk.PrintContext, page_nr: int) -> None: ...
    def get_body_font_name(self) -> str: ...
    def get_bottom_margin(self, unit: Gtk.Unit) -> float: ...
    def get_buffer(self) -> Buffer: ...
    def get_footer_font_name(self) -> str: ...
    def get_header_font_name(self) -> str: ...
    def get_highlight_syntax(self) -> bool: ...
    def get_left_margin(self, unit: Gtk.Unit) -> float: ...
    def get_line_numbers_font_name(self) -> str: ...
    def get_n_pages(self) -> int: ...
    def get_pagination_progress(self) -> float: ...
    def get_print_footer(self) -> bool: ...
    def get_print_header(self) -> bool: ...
    def get_print_line_numbers(self) -> int: ...
    def get_right_margin(self, unit: Gtk.Unit) -> float: ...
    def get_tab_width(self) -> int: ...
    def get_top_margin(self, unit: Gtk.Unit) -> float: ...
    def get_wrap_mode(self) -> Gtk.WrapMode: ...
    @classmethod
    def new(cls, buffer: Buffer) -> PrintCompositor: ...
    @classmethod
    def new_from_view(cls, view: View) -> PrintCompositor: ...
    def paginate(self, context: Gtk.PrintContext) -> bool: ...
    def set_body_font_name(self, font_name: str) -> None: ...
    def set_bottom_margin(self, margin: float, unit: Gtk.Unit) -> None: ...
    def set_footer_font_name(self, font_name: Optional[str] = None) -> None: ...
    def set_footer_format(
        self,
        separator: bool,
        left: Optional[str] = None,
        center: Optional[str] = None,
        right: Optional[str] = None,
    ) -> None: ...
    def set_header_font_name(self, font_name: Optional[str] = None) -> None: ...
    def set_header_format(
        self,
        separator: bool,
        left: Optional[str] = None,
        center: Optional[str] = None,
        right: Optional[str] = None,
    ) -> None: ...
    def set_highlight_syntax(self, highlight: bool) -> None: ...
    def set_left_margin(self, margin: float, unit: Gtk.Unit) -> None: ...
    def set_line_numbers_font_name(self, font_name: Optional[str] = None) -> None: ...
    def set_print_footer(self, print_: bool) -> None: ...
    def set_print_header(self, print_: bool) -> None: ...
    def set_print_line_numbers(self, interval: int) -> None: ...
    def set_right_margin(self, margin: float, unit: Gtk.Unit) -> None: ...
    def set_tab_width(self, width: int) -> None: ...
    def set_top_margin(self, margin: float, unit: Gtk.Unit) -> None: ...
    def set_wrap_mode(self, wrap_mode: Gtk.WrapMode) -> None: ...

class PrintCompositorClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class PrintCompositorPrivate(GObject.GPointer): ...

class Region(GObject.Object):
    class Props:
        buffer: Gtk.TextBuffer

    props: Props = ...
    parent_instance: GObject.Object = ...
    def __init__(self, buffer: Gtk.TextBuffer = ...): ...
    def add_region(self, region_to_add: Optional[Region] = None) -> None: ...
    def add_subregion(self, _start: Gtk.TextIter, _end: Gtk.TextIter) -> None: ...
    def get_bounds(self) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter]: ...
    def get_buffer(self) -> Optional[Gtk.TextBuffer]: ...
    def get_start_region_iter(self) -> RegionIter: ...
    def intersect_region(
        self, region2: Optional[Region] = None
    ) -> Optional[Region]: ...
    def intersect_subregion(
        self, _start: Gtk.TextIter, _end: Gtk.TextIter
    ) -> Optional[Region]: ...
    def is_empty(self) -> bool: ...
    @classmethod
    def new(cls, buffer: Gtk.TextBuffer) -> Region: ...
    def subtract_region(self, region_to_subtract: Optional[Region] = None) -> None: ...
    def subtract_subregion(self, _start: Gtk.TextIter, _end: Gtk.TextIter) -> None: ...
    def to_string(self) -> Optional[str]: ...

class RegionClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class RegionIter(GObject.GPointer):
    dummy1: None = ...
    dummy2: int = ...
    dummy3: None = ...
    def get_subregion(self) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter]: ...
    def is_end(self) -> bool: ...
    def next(self) -> bool: ...

class SearchContext(GObject.Object):
    class Props:
        buffer: Buffer
        highlight: bool
        match_style: Style
        occurrences_count: int
        regex_error: None
        settings: SearchSettings

    props: Props = ...
    parent: GObject.Object = ...
    priv: SearchContextPrivate = ...
    def __init__(
        self,
        buffer: Buffer = ...,
        highlight: bool = ...,
        match_style: Style = ...,
        settings: SearchSettings = ...,
    ): ...
    def backward(
        self, iter: Gtk.TextIter
    ) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter, bool]: ...
    def backward_async(
        self,
        iter: Gtk.TextIter,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def backward_finish(
        self, result: Gio.AsyncResult
    ) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter, bool]: ...
    def forward(
        self, iter: Gtk.TextIter
    ) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter, bool]: ...
    def forward_async(
        self,
        iter: Gtk.TextIter,
        cancellable: Optional[Gio.Cancellable] = None,
        callback: Optional[Callable[..., None]] = None,
        *user_data: Any,
    ) -> None: ...
    def forward_finish(
        self, result: Gio.AsyncResult
    ) -> Tuple[bool, Gtk.TextIter, Gtk.TextIter, bool]: ...
    def get_buffer(self) -> Buffer: ...
    def get_highlight(self) -> bool: ...
    def get_match_style(self) -> Style: ...
    def get_occurrence_position(
        self, match_start: Gtk.TextIter, match_end: Gtk.TextIter
    ) -> int: ...
    def get_occurrences_count(self) -> int: ...
    def get_regex_error(self) -> Optional[GLib.Error]: ...
    def get_settings(self) -> SearchSettings: ...
    @classmethod
    def new(
        cls, buffer: Buffer, settings: Optional[SearchSettings] = None
    ) -> SearchContext: ...
    def replace(
        self,
        match_start: Gtk.TextIter,
        match_end: Gtk.TextIter,
        replace: str,
        replace_length: int,
    ) -> bool: ...
    def replace_all(self, replace: str, replace_length: int) -> int: ...
    def set_highlight(self, highlight: bool) -> None: ...
    def set_match_style(self, match_style: Optional[Style] = None) -> None: ...

class SearchContextClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class SearchContextPrivate(GObject.GPointer): ...

class SearchSettings(GObject.Object):
    class Props:
        at_word_boundaries: bool
        case_sensitive: bool
        regex_enabled: bool
        search_text: str
        wrap_around: bool

    props: Props = ...
    parent: GObject.Object = ...
    priv: SearchSettingsPrivate = ...
    def __init__(
        self,
        at_word_boundaries: bool = ...,
        case_sensitive: bool = ...,
        regex_enabled: bool = ...,
        search_text: str = ...,
        wrap_around: bool = ...,
    ): ...
    def get_at_word_boundaries(self) -> bool: ...
    def get_case_sensitive(self) -> bool: ...
    def get_regex_enabled(self) -> bool: ...
    def get_search_text(self) -> Optional[str]: ...
    def get_wrap_around(self) -> bool: ...
    @classmethod
    def new(cls) -> SearchSettings: ...
    def set_at_word_boundaries(self, at_word_boundaries: bool) -> None: ...
    def set_case_sensitive(self, case_sensitive: bool) -> None: ...
    def set_regex_enabled(self, regex_enabled: bool) -> None: ...
    def set_search_text(self, search_text: Optional[str] = None) -> None: ...
    def set_wrap_around(self, wrap_around: bool) -> None: ...

class SearchSettingsClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class SearchSettingsPrivate(GObject.GPointer): ...

class SpaceDrawer(GObject.Object):
    class Props:
        enable_matrix: bool
        matrix: GLib.Variant

    props: Props = ...
    parent: GObject.Object = ...
    priv: SpaceDrawerPrivate = ...
    def __init__(self, enable_matrix: bool = ..., matrix: GLib.Variant = ...): ...
    def bind_matrix_setting(
        self, settings: Gio.Settings, key: str, flags: Gio.SettingsBindFlags
    ) -> None: ...
    def get_enable_matrix(self) -> bool: ...
    def get_matrix(self) -> GLib.Variant: ...
    def get_types_for_locations(
        self, locations: SpaceLocationFlags
    ) -> SpaceTypeFlags: ...
    @classmethod
    def new(cls) -> SpaceDrawer: ...
    def set_enable_matrix(self, enable_matrix: bool) -> None: ...
    def set_matrix(self, matrix: Optional[GLib.Variant] = None) -> None: ...
    def set_types_for_locations(
        self, locations: SpaceLocationFlags, types: SpaceTypeFlags
    ) -> None: ...

class SpaceDrawerClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class SpaceDrawerPrivate(GObject.GPointer): ...

class Style(GObject.Object):
    class Props:
        background: str
        background_set: bool
        bold: bool
        bold_set: bool
        foreground: str
        foreground_set: bool
        italic: bool
        italic_set: bool
        line_background: str
        line_background_set: bool
        pango_underline: Pango.Underline
        scale: str
        scale_set: bool
        strikethrough: bool
        strikethrough_set: bool
        underline_color: str
        underline_color_set: bool
        underline_set: bool

    props: Props = ...
    def __init__(
        self,
        background: str = ...,
        background_set: bool = ...,
        bold: bool = ...,
        bold_set: bool = ...,
        foreground: str = ...,
        foreground_set: bool = ...,
        italic: bool = ...,
        italic_set: bool = ...,
        line_background: str = ...,
        line_background_set: bool = ...,
        pango_underline: Pango.Underline = ...,
        scale: str = ...,
        scale_set: bool = ...,
        strikethrough: bool = ...,
        strikethrough_set: bool = ...,
        underline_color: str = ...,
        underline_color_set: bool = ...,
        underline_set: bool = ...,
    ): ...
    def apply(self, tag: Gtk.TextTag) -> None: ...
    def copy(self) -> Style: ...

class StyleClass(GObject.GPointer): ...

class StyleScheme(GObject.Object):
    class Props:
        description: str
        filename: str
        id: str
        name: str

    props: Props = ...
    base: GObject.Object = ...
    priv: StyleSchemePrivate = ...
    def __init__(self, id: str = ...): ...
    def get_authors(self) -> Optional[list[str]]: ...
    def get_description(self) -> Optional[str]: ...
    def get_filename(self) -> Optional[str]: ...
    def get_id(self) -> str: ...
    def get_name(self) -> str: ...
    def get_style(self, style_id: str) -> Optional[Style]: ...

class StyleSchemeChooser(GObject.GInterface):
    def get_style_scheme(self) -> StyleScheme: ...
    def set_style_scheme(self, scheme: StyleScheme) -> None: ...

class StyleSchemeChooserButton(
    Gtk.Button,
    Atk.ImplementorIface,
    Gtk.Actionable,
    Gtk.Activatable,
    Gtk.Buildable,
    StyleSchemeChooser,
):
    class Props:
        always_show_image: bool
        image: Gtk.Widget
        image_position: Gtk.PositionType
        label: str
        relief: Gtk.ReliefStyle
        use_stock: bool
        use_underline: bool
        xalign: float
        yalign: float
        border_width: int
        child: Gtk.Widget
        resize_mode: Gtk.ResizeMode
        app_paintable: bool
        can_default: bool
        can_focus: bool
        composite_child: bool
        double_buffered: bool
        events: Gdk.EventMask
        expand: bool
        focus_on_click: bool
        halign: Gtk.Align
        has_default: bool
        has_focus: bool
        has_tooltip: bool
        height_request: int
        hexpand: bool
        hexpand_set: bool
        is_focus: bool
        margin: int
        margin_bottom: int
        margin_end: int
        margin_left: int
        margin_right: int
        margin_start: int
        margin_top: int
        name: str
        no_show_all: bool
        opacity: float
        parent: Gtk.Container
        receives_default: bool
        scale_factor: int
        sensitive: bool
        style: Gtk.Style
        tooltip_markup: str
        tooltip_text: str
        valign: Gtk.Align
        vexpand: bool
        vexpand_set: bool
        visible: bool
        width_request: int
        window: Gdk.Window
        action_name: str
        action_target: GLib.Variant
        related_action: Gtk.Action
        use_action_appearance: bool
        style_scheme: StyleScheme

    props: Props = ...
    parent: Gtk.Button = ...
    def __init__(
        self,
        always_show_image: bool = ...,
        image: Gtk.Widget = ...,
        image_position: Gtk.PositionType = ...,
        label: str = ...,
        relief: Gtk.ReliefStyle = ...,
        use_stock: bool = ...,
        use_underline: bool = ...,
        xalign: float = ...,
        yalign: float = ...,
        border_width: int = ...,
        child: Gtk.Widget = ...,
        resize_mode: Gtk.ResizeMode = ...,
        app_paintable: bool = ...,
        can_default: bool = ...,
        can_focus: bool = ...,
        double_buffered: bool = ...,
        events: Gdk.EventMask = ...,
        expand: bool = ...,
        focus_on_click: bool = ...,
        halign: Gtk.Align = ...,
        has_default: bool = ...,
        has_focus: bool = ...,
        has_tooltip: bool = ...,
        height_request: int = ...,
        hexpand: bool = ...,
        hexpand_set: bool = ...,
        is_focus: bool = ...,
        margin: int = ...,
        margin_bottom: int = ...,
        margin_end: int = ...,
        margin_left: int = ...,
        margin_right: int = ...,
        margin_start: int = ...,
        margin_top: int = ...,
        name: str = ...,
        no_show_all: bool = ...,
        opacity: float = ...,
        parent: Gtk.Container = ...,
        receives_default: bool = ...,
        sensitive: bool = ...,
        style: Gtk.Style = ...,
        tooltip_markup: str = ...,
        tooltip_text: str = ...,
        valign: Gtk.Align = ...,
        vexpand: bool = ...,
        vexpand_set: bool = ...,
        visible: bool = ...,
        width_request: int = ...,
        action_name: str = ...,
        action_target: GLib.Variant = ...,
        related_action: Gtk.Action = ...,
        use_action_appearance: bool = ...,
        style_scheme: StyleScheme = ...,
    ): ...
    @classmethod
    def new(cls) -> StyleSchemeChooserButton: ...

class StyleSchemeChooserButtonClass(GObject.GPointer):
    parent: Gtk.ButtonClass = ...
    padding: list[None] = ...

class StyleSchemeChooserInterface(GObject.GPointer):
    base_interface: GObject.TypeInterface = ...
    get_style_scheme: Callable[[StyleSchemeChooser], StyleScheme] = ...
    set_style_scheme: Callable[[StyleSchemeChooser, StyleScheme], None] = ...
    padding: list[None] = ...

class StyleSchemeChooserWidget(
    Gtk.Bin, Atk.ImplementorIface, Gtk.Buildable, StyleSchemeChooser
):
    class Props:
        border_width: int
        child: Gtk.Widget
        resize_mode: Gtk.ResizeMode
        app_paintable: bool
        can_default: bool
        can_focus: bool
        composite_child: bool
        double_buffered: bool
        events: Gdk.EventMask
        expand: bool
        focus_on_click: bool
        halign: Gtk.Align
        has_default: bool
        has_focus: bool
        has_tooltip: bool
        height_request: int
        hexpand: bool
        hexpand_set: bool
        is_focus: bool
        margin: int
        margin_bottom: int
        margin_end: int
        margin_left: int
        margin_right: int
        margin_start: int
        margin_top: int
        name: str
        no_show_all: bool
        opacity: float
        parent: Gtk.Container
        receives_default: bool
        scale_factor: int
        sensitive: bool
        style: Gtk.Style
        tooltip_markup: str
        tooltip_text: str
        valign: Gtk.Align
        vexpand: bool
        vexpand_set: bool
        visible: bool
        width_request: int
        window: Gdk.Window
        style_scheme: StyleScheme

    props: Props = ...
    parent: Gtk.Bin = ...
    def __init__(
        self,
        border_width: int = ...,
        child: Gtk.Widget = ...,
        resize_mode: Gtk.ResizeMode = ...,
        app_paintable: bool = ...,
        can_default: bool = ...,
        can_focus: bool = ...,
        double_buffered: bool = ...,
        events: Gdk.EventMask = ...,
        expand: bool = ...,
        focus_on_click: bool = ...,
        halign: Gtk.Align = ...,
        has_default: bool = ...,
        has_focus: bool = ...,
        has_tooltip: bool = ...,
        height_request: int = ...,
        hexpand: bool = ...,
        hexpand_set: bool = ...,
        is_focus: bool = ...,
        margin: int = ...,
        margin_bottom: int = ...,
        margin_end: int = ...,
        margin_left: int = ...,
        margin_right: int = ...,
        margin_start: int = ...,
        margin_top: int = ...,
        name: str = ...,
        no_show_all: bool = ...,
        opacity: float = ...,
        parent: Gtk.Container = ...,
        receives_default: bool = ...,
        sensitive: bool = ...,
        style: Gtk.Style = ...,
        tooltip_markup: str = ...,
        tooltip_text: str = ...,
        valign: Gtk.Align = ...,
        vexpand: bool = ...,
        vexpand_set: bool = ...,
        visible: bool = ...,
        width_request: int = ...,
        style_scheme: StyleScheme = ...,
    ): ...
    @classmethod
    def new(cls) -> StyleSchemeChooserWidget: ...

class StyleSchemeChooserWidgetClass(GObject.GPointer):
    parent: Gtk.BinClass = ...
    padding: list[None] = ...

class StyleSchemeClass(GObject.GPointer):
    base_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class StyleSchemeManager(GObject.Object):
    class Props:
        scheme_ids: list[str]
        search_path: list[str]

    props: Props = ...
    parent: GObject.Object = ...
    priv: StyleSchemeManagerPrivate = ...
    def __init__(self, search_path: Sequence[str] = ...): ...
    def append_search_path(self, path: str) -> None: ...
    def force_rescan(self) -> None: ...
    @staticmethod
    def get_default() -> StyleSchemeManager: ...
    def get_scheme(self, scheme_id: str) -> Optional[StyleScheme]: ...
    def get_scheme_ids(self) -> Optional[list[str]]: ...
    def get_search_path(self) -> list[str]: ...
    @classmethod
    def new(cls) -> StyleSchemeManager: ...
    def prepend_search_path(self, path: str) -> None: ...
    def set_search_path(self, path: Optional[Sequence[str]] = None) -> None: ...

class StyleSchemeManagerClass(GObject.GPointer):
    parent_class: GObject.ObjectClass = ...
    padding: list[None] = ...

class StyleSchemeManagerPrivate(GObject.GPointer): ...
class StyleSchemePrivate(GObject.GPointer): ...

class Tag(Gtk.TextTag):
    class Props:
        draw_spaces: bool
        draw_spaces_set: bool
        accumulative_margin: bool
        background: str
        background_full_height: bool
        background_full_height_set: bool
        background_gdk: Gdk.Color
        background_rgba: Gdk.RGBA
        background_set: bool
        direction: Gtk.TextDirection
        editable: bool
        editable_set: bool
        fallback: bool
        fallback_set: bool
        family: str
        family_set: bool
        font: str
        font_desc: Pango.FontDescription
        font_features: str
        font_features_set: bool
        foreground: str
        foreground_gdk: Gdk.Color
        foreground_rgba: Gdk.RGBA
        foreground_set: bool
        indent: int
        indent_set: bool
        invisible: bool
        invisible_set: bool
        justification: Gtk.Justification
        justification_set: bool
        language: str
        language_set: bool
        left_margin: int
        left_margin_set: bool
        letter_spacing: int
        letter_spacing_set: bool
        name: str
        paragraph_background: str
        paragraph_background_gdk: Gdk.Color
        paragraph_background_rgba: Gdk.RGBA
        paragraph_background_set: bool
        pixels_above_lines: int
        pixels_above_lines_set: bool
        pixels_below_lines: int
        pixels_below_lines_set: bool
        pixels_inside_wrap: int
        pixels_inside_wrap_set: bool
        right_margin: int
        right_margin_set: bool
        rise: int
        rise_set: bool
        scale: float
        scale_set: bool
        size: int
        size_points: float
        size_set: bool
        stretch: Pango.Stretch
        stretch_set: bool
        strikethrough: bool
        strikethrough_rgba: Gdk.RGBA
        strikethrough_rgba_set: bool
        strikethrough_set: bool
        style: Pango.Style
        style_set: bool
        tabs: Pango.TabArray
        tabs_set: bool
        underline: Pango.Underline
        underline_rgba: Gdk.RGBA
        underline_rgba_set: bool
        underline_set: bool
        variant: Pango.Variant
        variant_set: bool
        weight: int
        weight_set: bool
        wrap_mode: Gtk.WrapMode
        wrap_mode_set: bool

    props: Props = ...
    parent_instance: Gtk.TextTag = ...
    def __init__(
        self,
        draw_spaces: bool = ...,
        draw_spaces_set: bool = ...,
        accumulative_margin: bool = ...,
        background: str = ...,
        background_full_height: bool = ...,
        background_full_height_set: bool = ...,
        background_gdk: Gdk.Color = ...,
        background_rgba: Gdk.RGBA = ...,
        background_set: bool = ...,
        direction: Gtk.TextDirection = ...,
        editable: bool = ...,
        editable_set: bool = ...,
        fallback: bool = ...,
        fallback_set: bool = ...,
        family: str = ...,
        family_set: bool = ...,
        font: str = ...,
        font_desc: Pango.FontDescription = ...,
        font_features: str = ...,
        font_features_set: bool = ...,
        foreground: str = ...,
        foreground_gdk: Gdk.Color = ...,
        foreground_rgba: Gdk.RGBA = ...,
        foreground_set: bool = ...,
        indent: int = ...,
        indent_set: bool = ...,
        invisible: bool = ...,
        invisible_set: bool = ...,
        justification: Gtk.Justification = ...,
        justification_set: bool = ...,
        language: str = ...,
        language_set: bool = ...,
        left_margin: int = ...,
        left_margin_set: bool = ...,
        letter_spacing: int = ...,
        letter_spacing_set: bool = ...,
        name: str = ...,
        paragraph_background: str = ...,
        paragraph_background_gdk: Gdk.Color = ...,
        paragraph_background_rgba: Gdk.RGBA = ...,
        paragraph_background_set: bool = ...,
        pixels_above_lines: int = ...,
        pixels_above_lines_set: bool = ...,
        pixels_below_lines: int = ...,
        pixels_below_lines_set: bool = ...,
        pixels_inside_wrap: int = ...,
        pixels_inside_wrap_set: bool = ...,
        right_margin: int = ...,
        right_margin_set: bool = ...,
        rise: int = ...,
        rise_set: bool = ...,
        scale: float = ...,
        scale_set: bool = ...,
        size: int = ...,
        size_points: float = ...,
        size_set: bool = ...,
        stretch: Pango.Stretch = ...,
        stretch_set: bool = ...,
        strikethrough: bool = ...,
        strikethrough_rgba: Gdk.RGBA = ...,
        strikethrough_rgba_set: bool = ...,
        strikethrough_set: bool = ...,
        style: Pango.Style = ...,
        style_set: bool = ...,
        tabs: Pango.TabArray = ...,
        tabs_set: bool = ...,
        underline: Pango.Underline = ...,
        underline_rgba: Gdk.RGBA = ...,
        underline_rgba_set: bool = ...,
        underline_set: bool = ...,
        variant: Pango.Variant = ...,
        variant_set: bool = ...,
        weight: int = ...,
        weight_set: bool = ...,
        wrap_mode: Gtk.WrapMode = ...,
        wrap_mode_set: bool = ...,
    ): ...
    @classmethod
    def new(cls, name: Optional[str] = None) -> Tag: ...

class TagClass(GObject.GPointer):
    parent_class: Gtk.TextTagClass = ...
    padding: list[None] = ...

class UndoManager(GObject.GInterface):
    def begin_not_undoable_action(self) -> None: ...
    def can_redo(self) -> bool: ...
    def can_redo_changed(self) -> None: ...
    def can_undo(self) -> bool: ...
    def can_undo_changed(self) -> None: ...
    def end_not_undoable_action(self) -> None: ...
    def redo(self) -> None: ...
    def undo(self) -> None: ...

class UndoManagerIface(GObject.GPointer):
    parent: GObject.TypeInterface = ...
    can_undo: Callable[[UndoManager], bool] = ...
    can_redo: Callable[[UndoManager], bool] = ...
    undo: Callable[[UndoManager], None] = ...
    redo: Callable[[UndoManager], None] = ...
    begin_not_undoable_action: Callable[[UndoManager], None] = ...
    end_not_undoable_action: Callable[[UndoManager], None] = ...
    can_undo_changed: Callable[[UndoManager], None] = ...
    can_redo_changed: Callable[[UndoManager], None] = ...

class View(Gtk.TextView, Atk.ImplementorIface, Gtk.Buildable, Gtk.Scrollable):
    class Props:
        auto_indent: bool
        background_pattern: BackgroundPatternType
        completion: Completion
        highlight_current_line: bool
        indent_on_tab: bool
        indent_width: int
        insert_spaces_instead_of_tabs: bool
        right_margin_position: int
        show_line_marks: bool
        show_line_numbers: bool
        show_right_margin: bool
        smart_backspace: bool
        smart_home_end: SmartHomeEndType
        space_drawer: SpaceDrawer
        tab_width: int
        accepts_tab: bool
        bottom_margin: int
        buffer: Gtk.TextBuffer
        cursor_visible: bool
        editable: bool
        im_module: str
        indent: int
        input_hints: Gtk.InputHints
        input_purpose: Gtk.InputPurpose
        justification: Gtk.Justification
        left_margin: int
        monospace: bool
        overwrite: bool
        pixels_above_lines: int
        pixels_below_lines: int
        pixels_inside_wrap: int
        populate_all: bool
        right_margin: int
        tabs: Pango.TabArray
        top_margin: int
        wrap_mode: Gtk.WrapMode
        border_width: int
        child: Gtk.Widget
        resize_mode: Gtk.ResizeMode
        app_paintable: bool
        can_default: bool
        can_focus: bool
        composite_child: bool
        double_buffered: bool
        events: Gdk.EventMask
        expand: bool
        focus_on_click: bool
        halign: Gtk.Align
        has_default: bool
        has_focus: bool
        has_tooltip: bool
        height_request: int
        hexpand: bool
        hexpand_set: bool
        is_focus: bool
        margin: int
        margin_bottom: int
        margin_end: int
        margin_left: int
        margin_right: int
        margin_start: int
        margin_top: int
        name: str
        no_show_all: bool
        opacity: float
        parent: Gtk.Container
        receives_default: bool
        scale_factor: int
        sensitive: bool
        style: Gtk.Style
        tooltip_markup: str
        tooltip_text: str
        valign: Gtk.Align
        vexpand: bool
        vexpand_set: bool
        visible: bool
        width_request: int
        window: Gdk.Window
        hadjustment: Gtk.Adjustment
        hscroll_policy: Gtk.ScrollablePolicy
        vadjustment: Gtk.Adjustment
        vscroll_policy: Gtk.ScrollablePolicy

    props: Props = ...
    parent: Gtk.TextView = ...
    priv: ViewPrivate = ...
    def __init__(
        self,
        auto_indent: bool = ...,
        background_pattern: BackgroundPatternType = ...,
        highlight_current_line: bool = ...,
        indent_on_tab: bool = ...,
        indent_width: int = ...,
        insert_spaces_instead_of_tabs: bool = ...,
        right_margin_position: int = ...,
        show_line_marks: bool = ...,
        show_line_numbers: bool = ...,
        show_right_margin: bool = ...,
        smart_backspace: bool = ...,
        smart_home_end: SmartHomeEndType = ...,
        tab_width: int = ...,
        accepts_tab: bool = ...,
        bottom_margin: int = ...,
        buffer: Gtk.TextBuffer = ...,
        cursor_visible: bool = ...,
        editable: bool = ...,
        im_module: str = ...,
        indent: int = ...,
        input_hints: Gtk.InputHints = ...,
        input_purpose: Gtk.InputPurpose = ...,
        justification: Gtk.Justification = ...,
        left_margin: int = ...,
        monospace: bool = ...,
        overwrite: bool = ...,
        pixels_above_lines: int = ...,
        pixels_below_lines: int = ...,
        pixels_inside_wrap: int = ...,
        populate_all: bool = ...,
        right_margin: int = ...,
        tabs: Pango.TabArray = ...,
        top_margin: int = ...,
        wrap_mode: Gtk.WrapMode = ...,
        border_width: int = ...,
        child: Gtk.Widget = ...,
        resize_mode: Gtk.ResizeMode = ...,
        app_paintable: bool = ...,
        can_default: bool = ...,
        can_focus: bool = ...,
        double_buffered: bool = ...,
        events: Gdk.EventMask = ...,
        expand: bool = ...,
        focus_on_click: bool = ...,
        halign: Gtk.Align = ...,
        has_default: bool = ...,
        has_focus: bool = ...,
        has_tooltip: bool = ...,
        height_request: int = ...,
        hexpand: bool = ...,
        hexpand_set: bool = ...,
        is_focus: bool = ...,
        margin: int = ...,
        margin_bottom: int = ...,
        margin_end: int = ...,
        margin_left: int = ...,
        margin_right: int = ...,
        margin_start: int = ...,
        margin_top: int = ...,
        name: str = ...,
        no_show_all: bool = ...,
        opacity: float = ...,
        parent: Gtk.Container = ...,
        receives_default: bool = ...,
        sensitive: bool = ...,
        style: Gtk.Style = ...,
        tooltip_markup: str = ...,
        tooltip_text: str = ...,
        valign: Gtk.Align = ...,
        vexpand: bool = ...,
        vexpand_set: bool = ...,
        visible: bool = ...,
        width_request: int = ...,
        hadjustment: Gtk.Adjustment = ...,
        hscroll_policy: Gtk.ScrollablePolicy = ...,
        vadjustment: Gtk.Adjustment = ...,
        vscroll_policy: Gtk.ScrollablePolicy = ...,
    ): ...
    def do_line_mark_activated(self, iter: Gtk.TextIter, event: Gdk.Event) -> None: ...
    def do_move_lines(self, down: bool) -> None: ...
    def do_move_words(self, step: int) -> None: ...
    def do_redo(self) -> None: ...
    def do_show_completion(self) -> None: ...
    def do_undo(self) -> None: ...
    def get_auto_indent(self) -> bool: ...
    def get_background_pattern(self) -> BackgroundPatternType: ...
    # override
    def get_buffer(self) -> Buffer: ...
    def get_completion(self) -> Completion: ...
    def get_gutter(self, window_type: Gtk.TextWindowType) -> Gutter: ...
    def get_highlight_current_line(self) -> bool: ...
    def get_indent_on_tab(self) -> bool: ...
    def get_indent_width(self) -> int: ...
    def get_insert_spaces_instead_of_tabs(self) -> bool: ...
    def get_mark_attributes(self, category: str, priority: int) -> MarkAttributes: ...
    def get_right_margin_position(self) -> int: ...
    def get_show_line_marks(self) -> bool: ...
    def get_show_line_numbers(self) -> bool: ...
    def get_show_right_margin(self) -> bool: ...
    def get_smart_backspace(self) -> bool: ...
    def get_smart_home_end(self) -> SmartHomeEndType: ...
    def get_space_drawer(self) -> SpaceDrawer: ...
    def get_tab_width(self) -> int: ...
    def get_visual_column(self, iter: Gtk.TextIter) -> int: ...
    def indent_lines(self, start: Gtk.TextIter, end: Gtk.TextIter) -> None: ...
    @classmethod
    def new(cls) -> View: ...
    @classmethod
    def new_with_buffer(cls, buffer: Buffer) -> View: ...
    def set_auto_indent(self, enable: bool) -> None: ...
    def set_background_pattern(
        self, background_pattern: BackgroundPatternType
    ) -> None: ...
    def set_highlight_current_line(self, highlight: bool) -> None: ...
    def set_indent_on_tab(self, enable: bool) -> None: ...
    def set_indent_width(self, width: int) -> None: ...
    def set_insert_spaces_instead_of_tabs(self, enable: bool) -> None: ...
    def set_mark_attributes(
        self, category: str, attributes: MarkAttributes, priority: int
    ) -> None: ...
    def set_right_margin_position(self, pos: int) -> None: ...
    def set_show_line_marks(self, show: bool) -> None: ...
    def set_show_line_numbers(self, show: bool) -> None: ...
    def set_show_right_margin(self, show: bool) -> None: ...
    def set_smart_backspace(self, smart_backspace: bool) -> None: ...
    def set_smart_home_end(self, smart_home_end: SmartHomeEndType) -> None: ...
    def set_tab_width(self, width: int) -> None: ...
    def unindent_lines(self, start: Gtk.TextIter, end: Gtk.TextIter) -> None: ...

class ViewClass(GObject.GPointer):
    parent_class: Gtk.TextViewClass = ...
    undo: Callable[[View], None] = ...
    redo: Callable[[View], None] = ...
    line_mark_activated: Callable[[View, Gtk.TextIter, Gdk.Event], None] = ...
    show_completion: Callable[[View], None] = ...
    move_lines: Callable[[View, bool], None] = ...
    move_words: Callable[[View, int], None] = ...
    padding: list[None] = ...

class ViewPrivate(GObject.GPointer): ...

class CompletionActivation(GObject.GFlags):
    INTERACTIVE = 1
    NONE = 0
    USER_REQUESTED = 2

class FileSaverFlags(GObject.GFlags):
    CREATE_BACKUP = 4
    IGNORE_INVALID_CHARS = 1
    IGNORE_MODIFICATION_TIME = 2
    NONE = 0

class GutterRendererState(GObject.GFlags):
    CURSOR = 1
    NORMAL = 0
    PRELIT = 2
    SELECTED = 4

class SortFlags(GObject.GFlags):
    CASE_SENSITIVE = 1
    NONE = 0
    REMOVE_DUPLICATES = 4
    REVERSE_ORDER = 2

class SpaceLocationFlags(GObject.GFlags):
    ALL = 7
    INSIDE_TEXT = 2
    LEADING = 1
    NONE = 0
    TRAILING = 4

class SpaceTypeFlags(GObject.GFlags):
    ALL = 15
    NBSP = 8
    NEWLINE = 4
    NONE = 0
    SPACE = 1
    TAB = 2

class BackgroundPatternType(GObject.GEnum):
    GRID = 1
    NONE = 0

class BracketMatchType(GObject.GEnum):
    FOUND = 3
    NONE = 0
    NOT_FOUND = 2
    OUT_OF_RANGE = 1

class ChangeCaseType(GObject.GEnum):
    LOWER = 0
    TITLE = 3
    TOGGLE = 2
    UPPER = 1

class CompletionError(GObject.GEnum):
    ALREADY_BOUND = 0
    NOT_BOUND = 1
    @staticmethod
    def quark() -> int: ...

class CompressionType(GObject.GEnum):
    GZIP = 1
    NONE = 0

class FileLoaderError(GObject.GEnum):
    CONVERSION_FALLBACK = 2
    ENCODING_AUTO_DETECTION_FAILED = 1
    TOO_BIG = 0
    @staticmethod
    def quark() -> int: ...

class FileSaverError(GObject.GEnum):
    EXTERNALLY_MODIFIED = 1
    INVALID_CHARS = 0
    @staticmethod
    def quark() -> int: ...

class GutterRendererAlignmentMode(GObject.GEnum):
    CELL = 0
    FIRST = 1
    LAST = 2

class NewlineType(GObject.GEnum):
    CR = 1
    CR_LF = 2
    LF = 0

class SmartHomeEndType(GObject.GEnum):
    AFTER = 2
    ALWAYS = 3
    BEFORE = 1
    DISABLED = 0

class ViewGutterPosition(GObject.GEnum):
    LINES = -30
    MARKS = -20



from typing import Any
from typing import Callable
from typing import Literal
from typing import Optional
from typing import Sequence
from typing import Tuple
from typing import Type
from typing import TypeVar

from gi.repository import GdkPixbuf
from gi.repository import GLib
from gi.repository import GObject

EXPIRES_DEFAULT: int = -1
EXPIRES_NEVER: int = 0
VERSION_MAJOR: int = 0
VERSION_MICRO: int = 3
VERSION_MINOR: int = 8
_lock = ...  # FIXME Constant
_namespace: str = "Notify"
_version: str = "0.7"

def get_app_name() -> str: ...
def get_server_caps() -> list[str]: ...
def get_server_info() -> Tuple[bool, str, str, str, str]: ...
def init(app_name: Optional[str] = None) -> bool: ...
def is_initted() -> bool: ...
def set_app_name(app_name: str) -> None: ...
def uninit() -> None: ...

class Notification(GObject.Object):
    """
    :Constructors:

    ::

        Notification(**properties)
        new(summary:str, body:str=None, icon:str=None) -> Notify.Notification

    Object NotifyNotification

    Signals from NotifyNotification:
      closed ()

    Properties from NotifyNotification:
      id -> gint: ID
        The notification ID
      app-name -> gchararray: Application name
        The application name to use for this notification
      summary -> gchararray: Summary
        The summary text
      body -> gchararray: Message Body
        The message body text
      icon-name -> gchararray: Icon Name
        The icon filename or icon theme-compliant name
      closed-reason -> gint: Closed Reason
        The reason code for why the notification was closed

    Signals from GObject:
      notify (GParam)
    """

    class Props:
        app_name: Optional[str]
        body: str
        closed_reason: int
        icon_name: str
        id: int
        summary: str

    props: Props = ...
    parent_object: GObject.Object = ...
    priv: NotificationPrivate = ...
    def __init__(
        self,
        app_name: Optional[str] = ...,
        body: str = ...,
        icon_name: str = ...,
        id: int = ...,
        summary: str = ...,
    ): ...
    def add_action(
        self, action: str, label: str, callback: Callable[..., None], *user_data: Any
    ) -> None: ...
    def clear_actions(self) -> None: ...
    def clear_hints(self) -> None: ...
    def close(self) -> bool: ...
    def do_closed(self) -> None: ...
    def get_activation_token(self) -> Optional[str]: ...
    def get_closed_reason(self) -> int: ...
    @classmethod
    def new(
        cls, summary: str, body: Optional[str] = None, icon: Optional[str] = None
    ) -> Notification: ...
    def set_app_name(self, app_name: Optional[str] = None) -> None: ...
    def set_category(self, category: str) -> None: ...
    def set_hint(self, key: str, value: Optional[GLib.Variant] = None) -> None: ...
    def set_hint_byte(self, key: str, value: int) -> None: ...
    def set_hint_byte_array(self, key: str, value: Sequence[int]) -> None: ...
    def set_hint_double(self, key: str, value: float) -> None: ...
    def set_hint_int32(self, key: str, value: int) -> None: ...
    def set_hint_string(self, key: str, value: str) -> None: ...
    def set_hint_uint32(self, key: str, value: int) -> None: ...
    def set_icon_from_pixbuf(self, icon: GdkPixbuf.Pixbuf) -> None: ...
    def set_image_from_pixbuf(self, pixbuf: GdkPixbuf.Pixbuf) -> None: ...
    def set_timeout(self, timeout: int) -> None: ...
    def set_urgency(self, urgency: Urgency) -> None: ...
    def show(self) -> bool: ...
    def update(
        self, summary: str, body: Optional[str] = None, icon: Optional[str] = None
    ) -> bool: ...

class NotificationClass(GObject.GPointer):
    """
    :Constructors:

    ::

        NotificationClass()
    """

    parent_class: GObject.ObjectClass = ...
    closed: Callable[[Notification], None] = ...

class NotificationPrivate(GObject.GPointer): ...

class ClosedReason(GObject.GEnum):
    API_REQUEST = 3
    DISMISSED = 2
    EXPIRED = 1
    UNDEFIEND = 4
    UNSET = -1

class Urgency(GObject.GEnum):
    CRITICAL = 2
    LOW = 0
    NORMAL = 1



import typing

import enum

from gi.repository import Gio
from gi.repository import GLib
from gi.repository import GObject

T = typing.TypeVar("T")

_lock = ...  # FIXME Constant
_namespace: str = "Gly"
_version: str = "2"

def loader_error_quark() -> int: ...
def memory_format_has_alpha(memory_format: MemoryFormat) -> bool: ...
def memory_format_is_premultiplied(memory_format: MemoryFormat) -> bool: ...

class Cicp(GObject.GBoxed):
    """
    :Constructors:

    ::

        Cicp()
    """

    color_primaries: int = ...
    transfer_characteristics: int = ...
    matrix_coefficients: int = ...
    video_full_range_flag: int = ...
    def copy(self) -> Cicp: ...
    def free(self) -> None: ...

class Creator(GObject.Object):
    """
    :Constructors:

    ::

        Creator(**properties)
        new(mime_type:str) -> Gly.Creator

    Object GlyCreator

    Properties from GlyCreator:
      sandbox-selector -> GlySandboxSelector: sandbox-selector
      mime-type -> gchararray: mime-type

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        mime_type: str
        sandbox_selector: SandboxSelector

    props: Props = ...
    def __init__(
        self, mime_type: str = ..., sandbox_selector: SandboxSelector = ...
    ) -> None: ...
    def add_frame(
        self, width: int, height: int, memory_format: MemoryFormat, texture: GLib.Bytes
    ) -> NewFrame: ...
    def add_frame_with_stride(
        self,
        width: int,
        height: int,
        stride: int,
        memory_format: MemoryFormat,
        texture: GLib.Bytes,
    ) -> NewFrame: ...
    def add_metadata_key_value(self, key: str, value: str) -> bool: ...
    def create(self) -> typing.Optional[EncodedImage]: ...
    def create_async(
        self,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def create_finish(self, result: Gio.AsyncResult) -> EncodedImage: ...
    @classmethod
    def new(cls, mime_type: str) -> Creator: ...
    def set_encoding_compression(self, compression: int) -> bool: ...
    def set_encoding_quality(self, quality: int) -> bool: ...
    def set_sandbox_selector(self, sandbox_selector: SandboxSelector) -> bool: ...

class CreatorClass(GObject.GPointer):
    """
    :Constructors:

    ::

        CreatorClass()
    """

    parent_class: GObject.ObjectClass = ...

class EncodedImage(GObject.Object):
    """
    :Constructors:

    ::

        EncodedImage(**properties)

    Object GlyEncodedImage

    Properties from GlyEncodedImage:
      data -> GBytes: data

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        data: GLib.Bytes

    props: Props = ...
    def get_data(self) -> GLib.Bytes: ...

class EncodedImageClass(GObject.GPointer):
    """
    :Constructors:

    ::

        EncodedImageClass()
    """

    parent_class: GObject.ObjectClass = ...

class Frame(GObject.Object):
    """
    :Constructors:

    ::

        Frame(**properties)

    Object GlyFrame

    Signals from GObject:
      notify (GParam)
    """
    def get_buf_bytes(self) -> GLib.Bytes: ...
    def get_color_cicp(self) -> typing.Optional[Cicp]: ...
    def get_delay(self) -> int: ...
    def get_height(self) -> int: ...
    def get_memory_format(self) -> MemoryFormat: ...
    def get_stride(self) -> int: ...
    def get_width(self) -> int: ...

class FrameClass(GObject.GPointer):
    """
    :Constructors:

    ::

        FrameClass()
    """

    parent_class: GObject.ObjectClass = ...

class FrameRequest(GObject.Object):
    """
    :Constructors:

    ::

        FrameRequest(**properties)
        new() -> Gly.FrameRequest

    Object GlyFrameRequest

    Properties from GlyFrameRequest:
      scale-width -> guint: scale-width
      scale-height -> guint: scale-height
      loop-animation -> gboolean: loop-animation

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        loop_animation: bool
        scale_height: int
        scale_width: int

    props: Props = ...
    def __init__(self, loop_animation: bool = ...) -> None: ...
    @classmethod
    def new(cls) -> FrameRequest: ...
    def set_loop_animation(self, loop_animation: bool) -> None: ...
    def set_scale(self, width: int, height: int) -> None: ...

class FrameRequestClass(GObject.GPointer):
    """
    :Constructors:

    ::

        FrameRequestClass()
    """

    parent_class: GObject.ObjectClass = ...

class Image(GObject.Object):
    """
    :Constructors:

    ::

        Image(**properties)

    Object GlyImage

    Signals from GObject:
      notify (GParam)
    """
    def get_height(self) -> int: ...
    def get_metadata_key_value(self, key: str) -> typing.Optional[str]: ...
    def get_metadata_keys(self) -> list[str]: ...
    def get_mime_type(self) -> str: ...
    def get_specific_frame(self, frame_request: FrameRequest) -> Frame: ...
    def get_specific_frame_async(
        self,
        frame_request: FrameRequest,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def get_specific_frame_finish(self, result: Gio.AsyncResult) -> Frame: ...
    def get_transformation_orientation(self) -> int: ...
    def get_width(self) -> int: ...
    def next_frame(self) -> Frame: ...
    def next_frame_async(
        self,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def next_frame_finish(self, result: Gio.AsyncResult) -> Frame: ...

class ImageClass(GObject.GPointer):
    """
    :Constructors:

    ::

        ImageClass()
    """

    parent_class: GObject.ObjectClass = ...

class Loader(GObject.Object):
    """
    :Constructors:

    ::

        Loader(**properties)
        new(file:Gio.File) -> Gly.Loader
        new_for_bytes(bytes:GLib.Bytes) -> Gly.Loader
        new_for_stream(stream:Gio.InputStream) -> Gly.Loader

    Object GlyLoader

    Properties from GlyLoader:
      file -> GFile: file
      stream -> GInputStream: stream
      bytes -> GBytes: bytes
      cancellable -> GCancellable: cancellable
      sandbox-selector -> GlySandboxSelector: sandbox-selector
      memory-format-selection -> GlyMemoryFormatSelection: memory-format-selection
      apply-transformation -> gboolean: apply-transformation

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        apply_transformation: bool
        bytes: GLib.Bytes
        cancellable: Gio.Cancellable
        file: Gio.File
        memory_format_selection: MemoryFormatSelection
        sandbox_selector: SandboxSelector
        stream: Gio.InputStream

    props: Props = ...
    def __init__(
        self,
        apply_transformation: bool = ...,
        bytes: GLib.Bytes = ...,
        cancellable: Gio.Cancellable = ...,
        file: Gio.File = ...,
        memory_format_selection: MemoryFormatSelection = ...,
        sandbox_selector: SandboxSelector = ...,
        stream: Gio.InputStream = ...,
    ) -> None: ...
    @staticmethod
    def get_mime_types() -> list[str]: ...
    @staticmethod
    def get_mime_types_async(
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    @staticmethod
    def get_mime_types_finish(result: Gio.AsyncResult) -> list[str]: ...
    def load(self) -> Image: ...
    def load_async(
        self,
        cancellable: typing.Optional[Gio.Cancellable] = None,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def load_finish(self, result: Gio.AsyncResult) -> Image: ...
    @classmethod
    def new(cls, file: Gio.File) -> Loader: ...
    @classmethod
    def new_for_bytes(cls, bytes: GLib.Bytes) -> Loader: ...
    @classmethod
    def new_for_stream(cls, stream: Gio.InputStream) -> Loader: ...
    def set_accepted_memory_formats(
        self, memory_format_selection: MemoryFormatSelection
    ) -> None: ...
    def set_apply_transformations(self, apply_transformations: bool) -> None: ...
    def set_sandbox_selector(self, sandbox_selector: SandboxSelector) -> None: ...

class LoaderClass(GObject.GPointer):
    """
    :Constructors:

    ::

        LoaderClass()
    """

    parent_class: GObject.ObjectClass = ...

class NewFrame(GObject.Object):
    """
    :Constructors:

    ::

        NewFrame(**properties)

    Object GlyNewFrame

    Signals from GObject:
      notify (GParam)
    """
    def set_color_icc_profile(self, icc_profile: GLib.Bytes) -> bool: ...

class NewFrameClass(GObject.GPointer):
    """
    :Constructors:

    ::

        NewFrameClass()
    """

    parent_class: GObject.ObjectClass = ...

class MemoryFormatSelection(enum.IntFlag):
    A8B8G8R8 = 64
    A8R8G8B8 = 16
    A8R8G8B8_PREMULTIPLIED = 2
    B8G8R8 = 256
    B8G8R8A8 = 8
    B8G8R8A8_PREMULTIPLIED = 1
    G16 = 4194304
    G16A16 = 2097152
    G16A16_PREMULTIPLIED = 1048576
    G8 = 524288
    G8A8 = 262144
    G8A8_PREMULTIPLIED = 131072
    R16G16B16 = 512
    R16G16B16A16 = 2048
    R16G16B16A16_FLOAT = 8192
    R16G16B16A16_PREMULTIPLIED = 1024
    R16G16B16_FLOAT = 4096
    R32G32B32A32_FLOAT = 65536
    R32G32B32A32_FLOAT_PREMULTIPLIED = 32768
    R32G32B32_FLOAT = 16384
    R8G8B8 = 128
    R8G8B8A8 = 32
    R8G8B8A8_PREMULTIPLIED = 4

class LoaderError(enum.IntEnum):
    FAILED = 0
    NO_MORE_FRAMES = 2
    UNKNOWN_IMAGE_FORMAT = 1
    @staticmethod
    def quark() -> int: ...

class MemoryFormat(enum.IntEnum):
    A8B8G8R8 = 6
    A8R8G8B8 = 4
    A8R8G8B8_PREMULTIPLIED = 1
    B8G8R8 = 8
    B8G8R8A8 = 3
    B8G8R8A8_PREMULTIPLIED = 0
    G16 = 22
    G16A16 = 21
    G16A16_PREMULTIPLIED = 20
    G8 = 19
    G8A8 = 18
    G8A8_PREMULTIPLIED = 17
    R16G16B16 = 9
    R16G16B16A16 = 11
    R16G16B16A16_FLOAT = 13
    R16G16B16A16_PREMULTIPLIED = 10
    R16G16B16_FLOAT = 12
    R32G32B32A32_FLOAT = 16
    R32G32B32A32_FLOAT_PREMULTIPLIED = 15
    R32G32B32_FLOAT = 14
    R8G8B8 = 7
    R8G8B8A8 = 5
    R8G8B8A8_PREMULTIPLIED = 2
    @staticmethod
    def has_alpha(memory_format: MemoryFormat) -> bool: ...
    @staticmethod
    def is_premultiplied(memory_format: MemoryFormat) -> bool: ...

class SandboxSelector(enum.IntEnum):
    AUTO = 0
    BWRAP = 1
    FLATPAK_SPAWN = 2
    NOT_SANDBOXED = 3



import typing

__version__: str
version_info: typing.Tuple[int, int, int]

def check_version(version: str) -> None: ...
def require_version(namespace: str, version: str) -> None: ...
def require_versions(versions: dict[str, str]) -> None: ...
def get_required_version(namespace: str) -> typing.Optional[str]: ...
def require_foreign(namespace: str, symbol: typing.Optional[str] = None) -> None: ...
def get_option(name: str) -> bool: ...
def disable_legacy_autoinit() -> None: ...



import typing

from gi.repository import DBus
from gi.repository import GLib
from gi.repository import GObject

T = typing.TypeVar("T")

COMPONENTLAYER_COUNT: int = 9
COORD_TYPE_COUNT: int = 3
DBUS_INTERFACE_ACCESSIBLE: str = "org.a11y.atspi.Accessible"
DBUS_INTERFACE_ACTION: str = "org.a11y.atspi.Action"
DBUS_INTERFACE_APPLICATION: str = "org.a11y.atspi.Application"
DBUS_INTERFACE_CACHE: str = "org.a11y.atspi.Cache"
DBUS_INTERFACE_COLLECTION: str = "org.a11y.atspi.Collection"
DBUS_INTERFACE_COMPONENT: str = "org.a11y.atspi.Component"
DBUS_INTERFACE_DEC: str = "org.a11y.atspi.DeviceEventController"
DBUS_INTERFACE_DEVICE_EVENT_LISTENER: str = "org.a11y.atspi.DeviceEventListener"
DBUS_INTERFACE_DOCUMENT: str = "org.a11y.atspi.Document"
DBUS_INTERFACE_EDITABLE_TEXT: str = "org.a11y.atspi.EditableText"
DBUS_INTERFACE_EVENT_KEYBOARD: str = "org.a11y.atspi.Event.Keyboard"
DBUS_INTERFACE_EVENT_MOUSE: str = "org.a11y.atspi.Event.Mouse"
DBUS_INTERFACE_EVENT_OBJECT: str = "org.a11y.atspi.Event.Object"
DBUS_INTERFACE_EVENT_SCREEN_READER: str = "org.a11y.atspi.Event.ScreenReader"
DBUS_INTERFACE_HYPERLINK: str = "org.a11y.atspi.Hyperlink"
DBUS_INTERFACE_HYPERTEXT: str = "org.a11y.atspi.Hypertext"
DBUS_INTERFACE_IMAGE: str = "org.a11y.atspi.Image"
DBUS_INTERFACE_REGISTRY: str = "org.a11y.atspi.Registry"
DBUS_INTERFACE_SELECTION: str = "org.a11y.atspi.Selection"
DBUS_INTERFACE_SOCKET: str = "org.a11y.atspi.Socket"
DBUS_INTERFACE_TABLE: str = "org.a11y.atspi.Table"
DBUS_INTERFACE_TABLE_CELL: str = "org.a11y.atspi.TableCell"
DBUS_INTERFACE_TEXT: str = "org.a11y.atspi.Text"
DBUS_INTERFACE_VALUE: str = "org.a11y.atspi.Value"
DBUS_NAME_REGISTRY: str = "org.a11y.atspi.Registry"
DBUS_PATH_DEC: str = "/org/a11y/atspi/registry/deviceeventcontroller"
DBUS_PATH_NULL: str = "/org/a11y/atspi/null"
DBUS_PATH_REGISTRY: str = "/org/a11y/atspi/registry"
DBUS_PATH_ROOT: str = "/org/a11y/atspi/accessible/root"
DBUS_PATH_SCREEN_READER: str = "/org/a11y/atspi/screenreader"
EVENTTYPE_COUNT: int = 4
KEYEVENTTYPE_COUNT: int = 2
KEYSYNTHTYPE_COUNT: int = 5
LOCALE_TYPE_COUNT: int = 6
MATCHTYPES_COUNT: int = 6
MODIFIERTYPE_COUNT: int = 8
RELATIONTYPE_COUNT: int = 24
ROLE_COUNT: int = 131
SCROLLTYPE_COUNT: int = 7
SORTORDER_COUNT: int = 8
STATETYPE_COUNT: int = 42
TEXT_BOUNDARY_TYPE_COUNT: int = 7
TEXT_CLIP_TYPE_COUNT: int = 4
TREETRAVERSALTYPE_COUNT: int = 4
_lock = ...  # FIXME Constant
_namespace: str = "Atspi"
_version: str = "2.0"

def deregister_device_event_listener(
    listener: DeviceListener, filter: None
) -> bool: ...
def deregister_keystroke_listener(
    listener: DeviceListener,
    key_set: typing.Optional[typing.Sequence[KeyDefinition]],
    modmask: int,
    event_types: int,
) -> bool: ...
def event_main() -> None: ...
def event_quit() -> None: ...
def exit() -> int: ...
def generate_keyboard_event(
    keyval: int, keystring: typing.Optional[str], synth_type: KeySynthType
) -> bool: ...
def generate_mouse_event(x: int, y: int, name: str) -> bool: ...
def generate_mouse_event_async(
    x: int,
    y: int,
    name: str,
    callback: typing.Optional[typing.Callable[..., None]] = None,
    *callback_data: typing.Any,
) -> None: ...
def get_desktop(i: int) -> Accessible: ...
def get_desktop_count() -> int: ...
def get_desktop_list() -> list[Accessible]: ...
def get_version() -> typing.Tuple[int, int, int]: ...
def init() -> int: ...
def is_initialized() -> bool: ...
def register_device_event_listener(
    listener: DeviceListener, event_types: int, filter: None
) -> bool: ...
def register_keystroke_listener(
    listener: DeviceListener,
    key_set: typing.Optional[typing.Sequence[KeyDefinition]],
    modmask: int,
    event_types: int,
    sync_type: KeyListenerSyncType,
) -> bool: ...
def role_get_localized_name(role: Role) -> str: ...
def role_get_name(role: Role) -> str: ...
def set_main_context(cnx: GLib.MainContext) -> None: ...
def set_reference_window(accessible: Accessible) -> None: ...
def set_timeout(val: int, startup_time: int) -> None: ...

class Accessible(
    Object,
    Action,
    Collection,
    Component,
    Document,
    EditableText,
    Hypertext,
    Image,
    Selection,
    Table,
    TableCell,
    Text,
    Value,
):
    """
    :Constructors:

    ::

        Accessible(**properties)

    Object AtspiAccessible

    Signals from AtspiAccessible:
      region-changed (gint, gint)
      mode-changed (gint, gchararray)

    Signals from GObject:
      notify (GParam)
    """

    parent: Object = ...
    accessible_parent: Accessible = ...
    children: list[None] = ...
    role: Role = ...
    interfaces: int = ...
    name: str = ...
    description: str = ...
    states: StateSet = ...
    attributes: dict[None, None] = ...
    cached_properties: int = ...
    priv: AccessiblePrivate = ...
    def clear_cache(self) -> None: ...
    def clear_cache_single(self) -> None: ...
    def do_mode_changed(self, enabled: bool) -> None: ...
    def do_region_changed(self, current_offset: int, last_offset: int) -> None: ...
    def get_accessible_id(self) -> str: ...
    def get_action(self) -> Action: ...
    def get_action_iface(self) -> Action: ...
    def get_application(self) -> Accessible: ...
    def get_atspi_version(self) -> str: ...
    def get_attributes(self) -> dict[str, str]: ...
    def get_attributes_as_array(self) -> list[str]: ...
    def get_child_at_index(self, child_index: int) -> Accessible: ...
    def get_child_count(self) -> int: ...
    def get_collection(self) -> Collection: ...
    def get_collection_iface(self) -> Collection: ...
    def get_component(self) -> Component: ...
    def get_component_iface(self) -> Component: ...
    def get_description(self) -> str: ...
    def get_document(self) -> Document: ...
    def get_document_iface(self) -> Document: ...
    def get_editable_text(self) -> EditableText: ...
    def get_editable_text_iface(self) -> EditableText: ...
    def get_help_text(self) -> str: ...
    def get_hyperlink(self) -> Hyperlink: ...
    def get_hypertext(self) -> Hypertext: ...
    def get_hypertext_iface(self) -> Hypertext: ...
    def get_id(self) -> int: ...
    def get_image(self) -> Image: ...
    def get_image_iface(self) -> Image: ...
    def get_index_in_parent(self) -> int: ...
    def get_interfaces(self) -> list[str]: ...
    def get_localized_role_name(self) -> str: ...
    def get_name(self) -> str: ...
    def get_object_locale(self) -> str: ...
    def get_parent(self) -> typing.Optional[Accessible]: ...
    def get_process_id(self) -> int: ...
    def get_relation_set(self) -> list[Relation]: ...
    def get_role(self) -> Role: ...
    def get_role_name(self) -> str: ...
    def get_selection(self) -> Selection: ...
    def get_selection_iface(self) -> Selection: ...
    def get_state_set(self) -> StateSet: ...
    def get_table(self) -> Table: ...
    def get_table_cell(self) -> TableCell: ...
    def get_table_iface(self) -> Table: ...
    def get_text(self) -> Text: ...
    def get_text_iface(self) -> Text: ...
    def get_toolkit_name(self) -> str: ...
    def get_toolkit_version(self) -> str: ...
    def get_value(self) -> Value: ...
    def get_value_iface(self) -> Value: ...
    def set_cache_mask(self, mask: Cache) -> None: ...

class AccessibleClass(GObject.GPointer):
    """
    :Constructors:

    ::

        AccessibleClass()
    """

    parent_class: ObjectClass = ...
    region_changed: typing.Callable[[Accessible, int, int], None] = ...
    mode_changed: typing.Callable[[Accessible, bool], None] = ...

class AccessiblePrivate(GObject.GPointer): ...

class Action(GObject.GInterface):
    """
    Interface AtspiAction
    """

    def do_action(self, i: int) -> bool: ...
    def get_action_description(self, i: int) -> str: ...
    def get_action_name(self, i: int) -> str: ...
    def get_key_binding(self, i: int) -> str: ...
    def get_localized_name(self, i: int) -> str: ...
    def get_n_actions(self) -> int: ...

class Application(GObject.Object):
    """
    :Constructors:

    ::

        Application(**properties)

    Object AtspiApplication

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    hash: dict[None, None] = ...
    bus_name: str = ...
    bus: DBus.Connection = ...
    root: None = ...
    cache: Cache = ...
    toolkit_name: str = ...
    toolkit_version: str = ...
    atspi_version: str = ...
    time_added: None = ...

class ApplicationClass(GObject.GPointer):
    """
    :Constructors:

    ::

        ApplicationClass()
    """

    parent_class: GObject.ObjectClass = ...

class Collection(GObject.GInterface):
    """
    Interface AtspiCollection
    """

    def get_active_descendant(self) -> Accessible: ...
    def get_matches(
        self, rule: MatchRule, sortby: CollectionSortOrder, count: int, traverse: bool
    ) -> list[Accessible]: ...
    def get_matches_from(
        self,
        current_object: Accessible,
        rule: MatchRule,
        sortby: CollectionSortOrder,
        tree: CollectionTreeTraversalType,
        count: int,
        traverse: bool,
    ) -> list[Accessible]: ...
    def get_matches_to(
        self,
        current_object: Accessible,
        rule: MatchRule,
        sortby: CollectionSortOrder,
        tree: CollectionTreeTraversalType,
        limit_scope: bool,
        count: int,
        traverse: bool,
    ) -> list[Accessible]: ...
    def is_ancestor_of(self, test: Accessible) -> bool: ...

class Component(GObject.GInterface):
    """
    Interface AtspiComponent
    """

    def contains(self, x: int, y: int, ctype: CoordType) -> bool: ...
    def get_accessible_at_point(
        self, x: int, y: int, ctype: CoordType
    ) -> typing.Optional[Accessible]: ...
    def get_alpha(self) -> float: ...
    def get_extents(self, ctype: CoordType) -> Rect: ...
    def get_layer(self) -> ComponentLayer: ...
    def get_mdi_z_order(self) -> int: ...
    def get_position(self, ctype: CoordType) -> Point: ...
    def get_size(self) -> Point: ...
    def grab_focus(self) -> bool: ...
    def scroll_to(self, type: ScrollType) -> bool: ...
    def scroll_to_point(self, coords: CoordType, x: int, y: int) -> bool: ...
    def set_extents(
        self, x: int, y: int, width: int, height: int, ctype: CoordType
    ) -> bool: ...
    def set_position(self, x: int, y: int, ctype: CoordType) -> bool: ...
    def set_size(self, width: int, height: int) -> bool: ...

class Device(GObject.Object):
    """
    :Constructors:

    ::

        Device(**properties)
        new() -> Atspi.Device

    Object AtspiDevice

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    def add_key_grab(
        self,
        kd: KeyDefinition,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> int: ...
    def add_key_watcher(
        self,
        callback: typing.Optional[typing.Callable[..., None]] = None,
        *user_data: typing.Any,
    ) -> None: ...
    def do_add_key_grab(self, kd: KeyDefinition) -> bool: ...
    def do_generate_mouse_event(
        self, obj: Accessible, x: int, y: int, name: str
    ) -> None: ...
    def do_get_locked_modifiers(self) -> int: ...
    def do_get_modifier(self, keycode: int) -> int: ...
    def do_grab_keyboard(self) -> bool: ...
    def do_map_modifier(self, keycode: int) -> int: ...
    def do_remove_key_grab(self, id: int) -> None: ...
    def do_ungrab_keyboard(self) -> None: ...
    def do_unmap_modifier(self, keycode: int) -> None: ...
    def generate_mouse_event(
        self, obj: Accessible, x: int, y: int, name: str
    ) -> None: ...
    def get_grab_by_id(self, id: int) -> KeyDefinition: ...
    def get_locked_modifiers(self) -> int: ...
    def get_modifier(self, keycode: int) -> int: ...
    def grab_keyboard(self) -> bool: ...
    def map_modifier(self, keycode: int) -> int: ...
    @classmethod
    def new(cls) -> Device: ...
    def notify_key(
        self, pressed: bool, keycode: int, keysym: int, state: int, text: str
    ) -> bool: ...
    def remove_key_grab(self, id: int) -> None: ...
    def ungrab_keyboard(self) -> None: ...
    def unmap_modifier(self, keycode: int) -> None: ...

class DeviceClass(GObject.GPointer):
    """
    :Constructors:

    ::

        DeviceClass()
    """

    parent_class: GObject.ObjectClass = ...
    add_key_grab: typing.Callable[[Device, KeyDefinition], bool] = ...
    remove_key_grab: typing.Callable[[Device, int], None] = ...
    map_modifier: typing.Callable[[Device, int], int] = ...
    unmap_modifier: typing.Callable[[Device, int], None] = ...
    get_modifier: typing.Callable[[Device, int], int] = ...
    grab_keyboard: typing.Callable[[Device], bool] = ...
    ungrab_keyboard: typing.Callable[[Device], None] = ...
    get_locked_modifiers: typing.Callable[[Device], int] = ...
    generate_mouse_event: typing.Callable[
        [Device, Accessible, int, int, str], None
    ] = ...

class DeviceEvent(GObject.GBoxed):
    """
    :Constructors:

    ::

        DeviceEvent()
    """

    type: EventType = ...
    id: int = ...
    hw_code: int = ...
    modifiers: int = ...
    timestamp: int = ...
    event_string: str = ...
    is_text: bool = ...

class DeviceLegacy(Device):
    """
    :Constructors:

    ::

        DeviceLegacy(**properties)
        new() -> Atspi.DeviceLegacy

    Object AtspiDeviceLegacy

    Signals from GObject:
      notify (GParam)
    """

    parent: Device = ...
    @classmethod
    def new(cls) -> DeviceLegacy: ...

class DeviceLegacyClass(GObject.GPointer):
    """
    :Constructors:

    ::

        DeviceLegacyClass()
    """

    parent_class: DeviceClass = ...

class DeviceListener(GObject.Object):
    """
    :Constructors:

    ::

        DeviceListener(**properties)
        new(callback:Atspi.DeviceListenerCB, user_data=None) -> Atspi.DeviceListener

    Object AtspiDeviceListener

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    id: int = ...
    callbacks: list[None] = ...
    def add_callback(
        self, callback: typing.Callable[..., bool], *user_data: typing.Any
    ) -> None: ...
    def do_device_event(self, event: DeviceEvent) -> bool: ...
    @classmethod
    def new(
        cls, callback: typing.Callable[..., bool], *user_data: typing.Any
    ) -> DeviceListener: ...
    def remove_callback(self, callback: typing.Callable[..., bool]) -> None: ...

class DeviceListenerClass(GObject.GPointer):
    """
    :Constructors:

    ::

        DeviceListenerClass()
    """

    parent_class: GObject.ObjectClass = ...
    device_event: typing.Callable[[DeviceListener, DeviceEvent], bool] = ...

class DeviceX11(Device):
    """
    :Constructors:

    ::

        DeviceX11(**properties)
        new() -> Atspi.DeviceX11

    Object AtspiDeviceX11

    Signals from GObject:
      notify (GParam)
    """

    parent: Device = ...
    @classmethod
    def new(cls) -> DeviceX11: ...

class DeviceX11Class(GObject.GPointer):
    """
    :Constructors:

    ::

        DeviceX11Class()
    """

    parent_class: DeviceClass = ...

class Document(GObject.GInterface):
    """
    Interface AtspiDocument
    """

    def get_current_page_number(self) -> int: ...
    def get_document_attribute_value(self, attribute: str) -> str: ...
    def get_document_attributes(self) -> dict[str, str]: ...
    def get_locale(self) -> str: ...
    def get_page_count(self) -> int: ...
    def get_text_selections(self) -> list[TextSelection]: ...
    def set_text_selections(
        self, selections: typing.Sequence[TextSelection]
    ) -> bool: ...

class EditableText(GObject.GInterface):
    """
    Interface AtspiEditableText
    """

    def copy_text(self, start_pos: int, end_pos: int) -> bool: ...
    def cut_text(self, start_pos: int, end_pos: int) -> bool: ...
    def delete_text(self, start_pos: int, end_pos: int) -> bool: ...
    def insert_text(self, position: int, text: str, length: int) -> bool: ...
    def paste_text(self, position: int) -> bool: ...
    def set_text_contents(self, new_contents: str) -> bool: ...

class Event(GObject.GBoxed):
    """
    :Constructors:

    ::

        Event()
    """

    type: str = ...
    source: Accessible = ...
    detail1: int = ...
    detail2: int = ...
    any_data: typing.Any = ...
    sender: Accessible = ...
    @staticmethod
    def main() -> None: ...
    @staticmethod
    def quit() -> None: ...

class EventListener(GObject.Object):
    """
    :Constructors:

    ::

        EventListener(**properties)
        new(callback:Atspi.EventListenerCB, user_data=None) -> Atspi.EventListener

    Object AtspiEventListener

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    callback: typing.Callable[..., None] = ...
    user_data: None = ...
    cb_destroyed: typing.Callable[[None], None] = ...
    def deregister(self, event_type: str) -> bool: ...
    @staticmethod
    def deregister_from_callback(
        callback: typing.Callable[..., None], event_type: str, *user_data: typing.Any
    ) -> bool: ...
    @classmethod
    def new(
        cls, callback: typing.Callable[..., None], *user_data: typing.Any
    ) -> EventListener: ...
    def register(self, event_type: str) -> bool: ...
    @staticmethod
    def register_from_callback(
        callback: typing.Callable[..., None], event_type: str, *user_data: typing.Any
    ) -> bool: ...
    @staticmethod
    def register_from_callback_full(
        callback: typing.Optional[typing.Callable[..., None]],
        event_type: str,
        properties: typing.Sequence[str],
        *user_data: typing.Any,
    ) -> bool: ...
    @staticmethod
    def register_from_callback_with_app(
        callback: typing.Optional[typing.Callable[..., None]],
        event_type: str,
        properties: typing.Sequence[str],
        app: typing.Optional[Accessible] = None,
        *user_data: typing.Any,
    ) -> bool: ...
    def register_full(
        self, event_type: str, properties: typing.Optional[typing.Sequence[str]] = None
    ) -> bool: ...
    def register_with_app(
        self,
        event_type: str,
        properties: typing.Optional[typing.Sequence[str]] = None,
        app: typing.Optional[Accessible] = None,
    ) -> bool: ...

class EventListenerClass(GObject.GPointer):
    """
    :Constructors:

    ::

        EventListenerClass()
    """

    parent_class: GObject.ObjectClass = ...

class EventListenerMode(GObject.GPointer):
    """
    :Constructors:

    ::

        EventListenerMode()
    """

    synchronous: bool = ...
    preemptive: bool = ...
    global_: bool = ...

class Hyperlink(Object):
    """
    :Constructors:

    ::

        Hyperlink(**properties)

    Object AtspiHyperlink

    Signals from GObject:
      notify (GParam)
    """

    parent: Object = ...
    def get_end_index(self) -> int: ...
    def get_index_range(self) -> Range: ...
    def get_n_anchors(self) -> int: ...
    def get_object(self, i: int) -> Accessible: ...
    def get_start_index(self) -> int: ...
    def get_uri(self, i: int) -> str: ...
    def is_valid(self) -> bool: ...

class HyperlinkClass(GObject.GPointer):
    """
    :Constructors:

    ::

        HyperlinkClass()
    """

    parent_class: ObjectClass = ...

class Hypertext(GObject.GInterface):
    """
    Interface AtspiHypertext
    """

    def get_link(self, link_index: int) -> typing.Optional[Hyperlink]: ...
    def get_link_index(self, character_offset: int) -> int: ...
    def get_n_links(self) -> int: ...

class Image(GObject.GInterface):
    """
    Interface AtspiImage
    """

    def get_image_description(self) -> str: ...
    def get_image_extents(self, ctype: CoordType) -> Rect: ...
    def get_image_locale(self) -> str: ...
    def get_image_position(self, ctype: CoordType) -> Point: ...
    def get_image_size(self) -> Point: ...

class KeyDefinition(GObject.GBoxed):
    """
    :Constructors:

    ::

        KeyDefinition()
    """

    keycode: int = ...
    keysym: int = ...
    keystring: str = ...
    modifiers: int = ...

class KeySet(GObject.GPointer):
    """
    :Constructors:

    ::

        KeySet()
    """

    keysyms: int = ...
    keycodes: int = ...
    keystrings: str = ...
    len: int = ...

class MatchRule(GObject.Object):
    """
    :Constructors:

    ::

        MatchRule(**properties)
        new(states:Atspi.StateSet, statematchtype:Atspi.CollectionMatchType, attributes:dict, attributematchtype:Atspi.CollectionMatchType, roles:list, rolematchtype:Atspi.CollectionMatchType, interfaces:list, interfacematchtype:Atspi.CollectionMatchType, invert:bool) -> Atspi.MatchRule

    Object AtspiMatchRule

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    states: StateSet = ...
    statematchtype: CollectionMatchType = ...
    attributes: dict[None, None] = ...
    attributematchtype: CollectionMatchType = ...
    interfaces: list[None] = ...
    interfacematchtype: CollectionMatchType = ...
    roles: list[int] = ...
    rolematchtype: CollectionMatchType = ...
    invert: bool = ...
    @classmethod
    def new(
        cls,
        states: StateSet,
        statematchtype: CollectionMatchType,
        attributes: dict[str, str],
        attributematchtype: CollectionMatchType,
        roles: typing.Sequence[Role],
        rolematchtype: CollectionMatchType,
        interfaces: typing.Sequence[str],
        interfacematchtype: CollectionMatchType,
        invert: bool,
    ) -> MatchRule: ...

class MatchRuleClass(GObject.GPointer):
    """
    :Constructors:

    ::

        MatchRuleClass()
    """

    parent_class: GObject.ObjectClass = ...

class Object(GObject.Object):
    """
    :Constructors:

    ::

        Object(**properties)

    Object AtspiObject

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    app: Application = ...
    path: str = ...

class ObjectClass(GObject.GPointer):
    """
    :Constructors:

    ::

        ObjectClass()
    """

    parent_class: GObject.ObjectClass = ...

class Point(GObject.GBoxed):
    """
    :Constructors:

    ::

        Point()
    """

    x: int = ...
    y: int = ...
    def copy(self) -> Point: ...

class Range(GObject.GBoxed):
    """
    :Constructors:

    ::

        Range()
    """

    start_offset: int = ...
    end_offset: int = ...
    def copy(self) -> Range: ...

class Rect(GObject.GBoxed):
    """
    :Constructors:

    ::

        Rect()
    """

    x: int = ...
    y: int = ...
    width: int = ...
    height: int = ...
    def copy(self) -> Rect: ...

class Relation(GObject.Object):
    """
    :Constructors:

    ::

        Relation(**properties)

    Object AtspiRelation

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    relation_type: RelationType = ...
    targets: list[None] = ...
    def get_n_targets(self) -> int: ...
    def get_relation_type(self) -> RelationType: ...
    def get_target(self, i: int) -> Accessible: ...

class RelationClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RelationClass()
    """

    parent_class: GObject.ObjectClass = ...

class Selection(GObject.GInterface):
    """
    Interface AtspiSelection
    """

    def clear_selection(self) -> bool: ...
    def deselect_child(self, child_index: int) -> bool: ...
    def deselect_selected_child(self, selected_child_index: int) -> bool: ...
    def get_n_selected_children(self) -> int: ...
    def get_selected_child(self, selected_child_index: int) -> Accessible: ...
    def is_child_selected(self, child_index: int) -> bool: ...
    def select_all(self) -> bool: ...
    def select_child(self, child_index: int) -> bool: ...

class StateSet(GObject.Object):
    """
    :Constructors:

    ::

        StateSet(**properties)
        new(states:list) -> Atspi.StateSet

    Object AtspiStateSet

    Signals from GObject:
      notify (GParam)
    """

    parent: GObject.Object = ...
    accessible: None = ...
    states: int = ...
    def add(self, state: StateType) -> None: ...
    def compare(self, set2: StateSet) -> StateSet: ...
    def contains(self, state: StateType) -> bool: ...
    def equals(self, set2: StateSet) -> bool: ...
    def get_states(self) -> list[StateType]: ...
    def is_empty(self) -> bool: ...
    @classmethod
    def new(cls, states: typing.Sequence[StateType]) -> StateSet: ...
    def remove(self, state: StateType) -> None: ...
    def set_by_name(self, name: str, enabled: bool) -> None: ...

class StateSetClass(GObject.GPointer):
    """
    :Constructors:

    ::

        StateSetClass()
    """

    parent_class: GObject.ObjectClass = ...

class Table(GObject.GInterface):
    """
    Interface AtspiTable
    """

    def add_column_selection(self, column: int) -> bool: ...
    def add_row_selection(self, row: int) -> bool: ...
    def get_accessible_at(self, row: int, column: int) -> Accessible: ...
    def get_caption(self) -> Accessible: ...
    def get_column_at_index(self, index: int) -> int: ...
    def get_column_description(self, column: int) -> str: ...
    def get_column_extent_at(self, row: int, column: int) -> int: ...
    def get_column_header(self, column: int) -> Accessible: ...
    def get_index_at(self, row: int, column: int) -> int: ...
    def get_n_columns(self) -> int: ...
    def get_n_rows(self) -> int: ...
    def get_n_selected_columns(self) -> int: ...
    def get_n_selected_rows(self) -> int: ...
    def get_row_at_index(self, index: int) -> int: ...
    def get_row_column_extents_at_index(
        self, index: int
    ) -> typing.Tuple[bool, int, int, int, int, bool]: ...
    def get_row_description(self, row: int) -> str: ...
    def get_row_extent_at(self, row: int, column: int) -> int: ...
    def get_row_header(self, row: int) -> Accessible: ...
    def get_selected_columns(self) -> list[int]: ...
    def get_selected_rows(self) -> list[int]: ...
    def get_summary(self) -> Accessible: ...
    def is_column_selected(self, column: int) -> bool: ...
    def is_row_selected(self, row: int) -> bool: ...
    def is_selected(self, row: int, column: int) -> bool: ...
    def remove_column_selection(self, column: int) -> bool: ...
    def remove_row_selection(self, row: int) -> bool: ...

class TableCell(GObject.GInterface):
    """
    Interface AtspiTableCell
    """

    def get_column_header_cells(self) -> list[Accessible]: ...
    def get_column_index(self) -> int: ...
    def get_column_span(self) -> int: ...
    def get_position(self) -> typing.Tuple[int, int, int]: ...
    def get_row_column_span(self) -> typing.Tuple[int, int, int, int]: ...
    def get_row_header_cells(self) -> list[Accessible]: ...
    def get_row_span(self) -> int: ...
    def get_table(self) -> Accessible: ...

class Text(GObject.GInterface):
    """
    Interface AtspiText
    """

    def add_selection(self, start_offset: int, end_offset: int) -> bool: ...
    def get_attribute_run(
        self, offset: int, include_defaults: bool
    ) -> typing.Tuple[dict[str, str], int, int]: ...
    def get_bounded_ranges(
        self,
        x: int,
        y: int,
        width: int,
        height: int,
        type: CoordType,
        clipTypeX: TextClipType,
        clipTypeY: TextClipType,
    ) -> list[TextRange]: ...
    def get_caret_offset(self) -> int: ...
    def get_character_at_offset(self, offset: int) -> int: ...
    def get_character_count(self) -> int: ...
    def get_character_extents(self, offset: int, type: CoordType) -> Rect: ...
    def get_default_attributes(self) -> dict[str, str]: ...
    def get_n_selections(self) -> int: ...
    def get_offset_at_point(self, x: int, y: int, type: CoordType) -> int: ...
    def get_range_extents(
        self, start_offset: int, end_offset: int, type: CoordType
    ) -> Rect: ...
    def get_selection(self, selection_num: int) -> Range: ...
    def get_string_at_offset(
        self, offset: int, granularity: TextGranularity
    ) -> TextRange: ...
    def get_text(self, start_offset: int, end_offset: int) -> str: ...
    def get_text_after_offset(
        self, offset: int, type: TextBoundaryType
    ) -> TextRange: ...
    def get_text_at_offset(self, offset: int, type: TextBoundaryType) -> TextRange: ...
    def get_text_attribute_value(
        self, offset: int, attribute_name: str
    ) -> typing.Optional[str]: ...
    def get_text_attributes(
        self, offset: int
    ) -> typing.Tuple[dict[str, str], int, int]: ...
    def get_text_before_offset(
        self, offset: int, type: TextBoundaryType
    ) -> TextRange: ...
    def remove_selection(self, selection_num: int) -> bool: ...
    def scroll_substring_to(
        self, start_offset: int, end_offset: int, type: ScrollType
    ) -> bool: ...
    def scroll_substring_to_point(
        self, start_offset: int, end_offset: int, coords: CoordType, x: int, y: int
    ) -> bool: ...
    def set_caret_offset(self, new_offset: int) -> bool: ...
    def set_selection(
        self, selection_num: int, start_offset: int, end_offset: int
    ) -> bool: ...

class TextRange(GObject.GBoxed):
    """
    :Constructors:

    ::

        TextRange()
    """

    start_offset: int = ...
    end_offset: int = ...
    content: str = ...

class TextSelection(GObject.GPointer):
    """
    :Constructors:

    ::

        TextSelection()
    """

    start_object: Accessible = ...
    start_offset: int = ...
    end_object: Accessible = ...
    end_offset: int = ...
    start_is_active: bool = ...

class Value(GObject.GInterface):
    """
    Interface AtspiValue
    """

    def get_current_value(self) -> float: ...
    def get_maximum_value(self) -> float: ...
    def get_minimum_increment(self) -> float: ...
    def get_minimum_value(self) -> float: ...
    def get_text(self) -> str: ...
    def set_current_value(self, new_value: float) -> bool: ...

class Cache(GObject.GFlags):
    ALL = 1073741823
    ATTRIBUTES = 128
    CHILDREN = 2
    DEFAULT = 127
    DESCRIPTION = 8
    INTERFACES = 64
    NAME = 4
    NONE = 0
    PARENT = 1
    ROLE = 32
    STATES = 16
    UNDEFINED = 1073741824

class KeyListenerSyncType(GObject.GFlags):
    ALL_WINDOWS = 4
    CANCONSUME = 2
    NOSYNC = 0
    SYNCHRONOUS = 1

class CollectionMatchType(GObject.GEnum):
    ALL = 1
    ANY = 2
    EMPTY = 4
    INVALID = 0
    LAST_DEFINED = 5
    NONE = 3

class CollectionSortOrder(GObject.GEnum):
    CANONICAL = 1
    FLOW = 2
    INVALID = 0
    LAST_DEFINED = 7
    REVERSE_CANONICAL = 4
    REVERSE_FLOW = 5
    REVERSE_TAB = 6
    TAB = 3

class CollectionTreeTraversalType(GObject.GEnum):
    INORDER = 2
    LAST_DEFINED = 3
    RESTRICT_CHILDREN = 0
    RESTRICT_SIBLING = 1

class ComponentLayer(GObject.GEnum):
    BACKGROUND = 1
    CANVAS = 2
    INVALID = 0
    LAST_DEFINED = 8
    MDI = 4
    OVERLAY = 6
    POPUP = 5
    WIDGET = 3
    WINDOW = 7

class CoordType(GObject.GEnum):
    PARENT = 2
    SCREEN = 0
    WINDOW = 1

class EventType(GObject.GEnum):
    BUTTON_PRESSED_EVENT = 2
    BUTTON_RELEASED_EVENT = 3
    KEY_PRESSED_EVENT = 0
    KEY_RELEASED_EVENT = 1

class KeyEventType(GObject.GEnum):
    PRESSED = 0
    RELEASED = 1

class KeySynthType(GObject.GEnum):
    LOCKMODIFIERS = 5
    PRESS = 0
    PRESSRELEASE = 2
    RELEASE = 1
    STRING = 4
    SYM = 3
    UNLOCKMODIFIERS = 6

class Live(GObject.GEnum):
    ASSERTIVE = 2
    NONE = 0
    POLITE = 1

class LocaleType(GObject.GEnum):
    COLLATE = 1
    CTYPE = 2
    MESSAGES = 0
    MONETARY = 3
    NUMERIC = 4
    TIME = 5

class ModifierType(GObject.GEnum):
    ALT = 3
    CONTROL = 2
    META = 4
    META2 = 5
    META3 = 6
    NUMLOCK = 14
    SHIFT = 0
    SHIFTLOCK = 1

class RelationType(GObject.GEnum):
    CONTROLLED_BY = 4
    CONTROLLER_FOR = 3
    DESCRIBED_BY = 18
    DESCRIPTION_FOR = 17
    DETAILS = 19
    DETAILS_FOR = 20
    EMBEDDED_BY = 14
    EMBEDS = 13
    ERROR_FOR = 22
    ERROR_MESSAGE = 21
    EXTENDED = 9
    FLOWS_FROM = 11
    FLOWS_TO = 10
    LABELLED_BY = 2
    LABEL_FOR = 1
    LAST_DEFINED = 23
    MEMBER_OF = 5
    NODE_CHILD_OF = 7
    NODE_PARENT_OF = 8
    NULL = 0
    PARENT_WINDOW_OF = 16
    POPUP_FOR = 15
    SUBWINDOW_OF = 12
    TOOLTIP_FOR = 6

class Role(GObject.GEnum):
    ACCELERATOR_LABEL = 1
    ALERT = 2
    ANIMATION = 3
    APPLICATION = 75
    ARROW = 4
    ARTICLE = 109
    AUDIO = 106
    AUTOCOMPLETE = 76
    BLOCK_QUOTE = 105
    CALENDAR = 5
    CANVAS = 6
    CAPTION = 81
    CHART = 80
    CHECK_BOX = 7
    CHECK_MENU_ITEM = 8
    COLOR_CHOOSER = 9
    COLUMN_HEADER = 10
    COMBO_BOX = 11
    COMMENT = 97
    CONTENT_DELETION = 125
    CONTENT_INSERTION = 126
    DATE_EDITOR = 12
    DEFINITION = 108
    DESCRIPTION_LIST = 121
    DESCRIPTION_TERM = 122
    DESCRIPTION_VALUE = 123
    DESKTOP_FRAME = 14
    DESKTOP_ICON = 13
    DIAL = 15
    DIALOG = 16
    DIRECTORY_PANE = 17
    DOCUMENT_EMAIL = 96
    DOCUMENT_FRAME = 82
    DOCUMENT_PRESENTATION = 93
    DOCUMENT_SPREADSHEET = 92
    DOCUMENT_TEXT = 94
    DOCUMENT_WEB = 95
    DRAWING_AREA = 18
    EDITBAR = 77
    EMBEDDED = 78
    ENTRY = 79
    EXTENDED = 70
    FILE_CHOOSER = 19
    FILLER = 20
    FOCUS_TRAVERSABLE = 21
    FONT_CHOOSER = 22
    FOOTER = 72
    FOOTNOTE = 124
    FORM = 87
    FRAME = 23
    GLASS_PANE = 24
    GROUPING = 99
    HEADER = 71
    HEADING = 83
    HTML_CONTAINER = 25
    ICON = 26
    IMAGE = 27
    IMAGE_MAP = 100
    INFO_BAR = 102
    INPUT_METHOD_WINDOW = 89
    INTERNAL_FRAME = 28
    INVALID = 0
    LABEL = 29
    LANDMARK = 110
    LAST_DEFINED = 130
    LAYERED_PANE = 30
    LEVEL_BAR = 103
    LINK = 88
    LIST = 31
    LIST_BOX = 98
    LIST_ITEM = 32
    LOG = 111
    MARK = 127
    MARQUEE = 112
    MATH = 113
    MATH_FRACTION = 117
    MATH_ROOT = 118
    MENU = 33
    MENU_BAR = 34
    MENU_ITEM = 35
    NOTIFICATION = 101
    OPTION_PANE = 36
    PAGE = 84
    PAGE_TAB = 37
    PAGE_TAB_LIST = 38
    PANEL = 39
    PARAGRAPH = 73
    PASSWORD_TEXT = 40
    POPUP_MENU = 41
    PROGRESS_BAR = 42
    PUSH_BUTTON = 43
    PUSH_BUTTON_MENU = 129
    RADIO_BUTTON = 44
    RADIO_MENU_ITEM = 45
    RATING = 114
    REDUNDANT_OBJECT = 86
    ROOT_PANE = 46
    ROW_HEADER = 47
    RULER = 74
    SCROLL_BAR = 48
    SCROLL_PANE = 49
    SECTION = 85
    SEPARATOR = 50
    SLIDER = 51
    SPIN_BUTTON = 52
    SPLIT_PANE = 53
    STATIC = 116
    STATUS_BAR = 54
    SUBSCRIPT = 119
    SUGGESTION = 128
    SUPERSCRIPT = 120
    TABLE = 55
    TABLE_CELL = 56
    TABLE_COLUMN_HEADER = 57
    TABLE_ROW = 90
    TABLE_ROW_HEADER = 58
    TEAROFF_MENU_ITEM = 59
    TERMINAL = 60
    TEXT = 61
    TIMER = 115
    TITLE_BAR = 104
    TOGGLE_BUTTON = 62
    TOOL_BAR = 63
    TOOL_TIP = 64
    TREE = 65
    TREE_ITEM = 91
    TREE_TABLE = 66
    UNKNOWN = 67
    VIDEO = 107
    VIEWPORT = 68
    WINDOW = 69
    @staticmethod
    def get_localized_name(role: Role) -> str: ...
    @staticmethod
    def get_name(role: Role) -> str: ...

class ScrollType(GObject.GEnum):
    ANYWHERE = 6
    BOTTOM_EDGE = 3
    BOTTOM_RIGHT = 1
    LEFT_EDGE = 4
    RIGHT_EDGE = 5
    TOP_EDGE = 2
    TOP_LEFT = 0

class StateType(GObject.GEnum):
    ACTIVE = 1
    ANIMATED = 35
    ARMED = 2
    BUSY = 3
    CHECKABLE = 41
    CHECKED = 4
    COLLAPSED = 5
    DEFUNCT = 6
    EDITABLE = 7
    ENABLED = 8
    EXPANDABLE = 9
    EXPANDED = 10
    FOCUSABLE = 11
    FOCUSED = 12
    HAS_POPUP = 42
    HAS_TOOLTIP = 13
    HORIZONTAL = 14
    ICONIFIED = 15
    INDETERMINATE = 32
    INVALID = 0
    INVALID_ENTRY = 36
    IS_DEFAULT = 39
    LAST_DEFINED = 44
    MANAGES_DESCENDANTS = 31
    MODAL = 16
    MULTISELECTABLE = 18
    MULTI_LINE = 17
    OPAQUE = 19
    PRESSED = 20
    READ_ONLY = 43
    REQUIRED = 33
    RESIZABLE = 21
    SELECTABLE = 22
    SELECTABLE_TEXT = 38
    SELECTED = 23
    SENSITIVE = 24
    SHOWING = 25
    SINGLE_LINE = 26
    STALE = 27
    SUPPORTS_AUTOCOMPLETION = 37
    TRANSIENT = 28
    TRUNCATED = 34
    VERTICAL = 29
    VISIBLE = 30
    VISITED = 40

class TextBoundaryType(GObject.GEnum):
    CHAR = 0
    LINE_END = 6
    LINE_START = 5
    SENTENCE_END = 4
    SENTENCE_START = 3
    WORD_END = 2
    WORD_START = 1

class TextClipType(GObject.GEnum):
    BOTH = 3
    MAX = 2
    MIN = 1
    NONE = 0

class TextGranularity(GObject.GEnum):
    CHAR = 0
    LINE = 3
    PARAGRAPH = 4
    SENTENCE = 2
    WORD = 1
